@@ -18,7 +18,6 @@
  */
 
 #include <gtk/gtk.h>
-#include <glib/gunicode.h>
 
 #include <string.h>
 #include "gtkcharsel.h"
@@ -160,11 +159,11 @@ gtk_char_selection_init (GtkCharSelection *charsel)
   gtk_box_pack_end (GTK_BOX (main_vbox), action_area, FALSE, FALSE, 0);
   gtk_widget_show (action_area);
 
-  charsel->ok_button = gtk_button_new_from_stock (GTK_STOCK_OK);
+  charsel->ok_button = gtk_button_new_with_label ("OK");
   gtk_box_pack_start (GTK_BOX (action_area), charsel->ok_button, TRUE, TRUE, 0);
   gtk_widget_show (charsel->ok_button);
 
-  charsel->cancel_button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+  charsel->cancel_button = gtk_button_new_with_label ("Cancel");
   gtk_box_pack_start (GTK_BOX (action_area), charsel->cancel_button, TRUE, TRUE, 0);
   gtk_widget_show (charsel->cancel_button);
 
@@ -206,61 +205,41 @@ new_font(GtkFontCombo *font_combo, gpointer data)
   GdkColor color;
   GdkPixmap *pixmap;
   GtkWidget *wpixmap;
-  PangoFontDescription *font;
-  PangoContext *context = gtk_widget_get_pango_context(GTK_WIDGET(data));
-  PangoFontMetrics *metrics;
-  PangoLayout *layout;
-  gint ascent, descent;
-  gint width;
   gint i;
  
   charsel = GTK_CHAR_SELECTION(data);
  
   gdk_color_white(gtk_widget_get_colormap(GTK_WIDGET(charsel)), &color);
 
-  font = gtk_font_combo_get_font_description(font_combo);
-  layout = gtk_widget_create_pango_layout(GTK_WIDGET(charsel), NULL);
-  pango_layout_set_font_description(layout, font);
-  metrics = pango_context_get_metrics(context, font, pango_context_get_language(context));
-  ascent = PANGO_PIXELS(pango_font_metrics_get_ascent(metrics));
-  descent = PANGO_PIXELS(pango_font_metrics_get_descent(metrics));
-  pango_font_metrics_unref(metrics);
-  width = ascent + descent + 8;
-
   for(i = 0; i < 256; i++){
     GtkWidget *widget;
-    PangoRectangle rect;
     GtkRequisition req;
-    gchar *s2;
-    gunichar s[2];
+    gint width;
+    gchar s[2];
     s[0] = i;
     s[1] = '\0';
 
-    s2 = g_ucs4_to_utf8(s,1,NULL,NULL,NULL);
-
     widget = GTK_WIDGET(charsel->button[i]);
 
     if(GTK_BIN(widget)->child)
       gtk_container_remove(GTK_CONTAINER(widget), GTK_BIN(widget)->child);
 
-    pango_layout_set_text(layout, s2, -1);
-    g_free(s2);
-    pango_layout_get_extents(layout, NULL, &rect);
-
-    req.width = width + 2 * widget->style->xthickness;
-    req.height = PANGO_PIXELS(rect.height);
+    req.width = gdk_char_width_wc(font_combo->font, s[0]);
+    req.height = font_combo->font->ascent + font_combo->font->descent;
+    width = MAX(req.width + 8, 3 * req.height / 2);
 
     if(GTK_WIDGET_MAPPED(widget)){
       pixmap = gdk_pixmap_new(widget->window, width, width, -1);
       gdk_draw_rectangle(pixmap, widget->style->white_gc, TRUE, 0, 0, width, width);
-      gdk_draw_layout(pixmap, widget->style->fg_gc[0], width/2 - PANGO_PIXELS(rect.width)/2, descent, layout);
+      gdk_draw_text(pixmap, font_combo->font, widget->style->fg_gc[0], width/2 - req.width/2, width/2 + (font_combo->font->ascent - font_combo->font->descent)/2, s, 1);
       wpixmap = gtk_pixmap_new(pixmap, NULL);
       gtk_container_add (GTK_CONTAINER (charsel->button[i]), wpixmap);
       gtk_widget_show(wpixmap);
       gdk_pixmap_unref(pixmap);
     }
 
-    gtk_widget_set_usize(widget, req.width, req.width);
+    width += 2 * (GTK_CONTAINER(widget)->border_width + widget->style->klass->xthickness);
+    gtk_widget_set_usize(widget, width, width);
 
     if(charsel->selection == i)
       gtk_toggle_button_set_active(charsel->button[i], TRUE);
@@ -268,8 +247,6 @@ new_font(GtkFontCombo *font_combo, gpointer data)
       gtk_toggle_button_set_active(charsel->button[i], FALSE);
   }
 
-  pango_font_description_free(font);
-  g_object_unref(G_OBJECT(layout));
 }
 
 static void 
@@ -289,7 +266,9 @@ new_selection(GtkButton *button, gpointer data)
   }
 
   if(new_selection == charsel->selection){
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(charsel->button[new_selection]), TRUE);
+      GTK_BUTTON(charsel->button[new_selection])->button_down=TRUE;
+      GTK_TOGGLE_BUTTON(charsel->button[new_selection])->active=TRUE;
+      gtk_widget_set_state(GTK_WIDGET(charsel->button[new_selection]), GTK_STATE_ACTIVE);
 
       return;
   }
@@ -311,7 +290,10 @@ gtk_char_selection_set_selection(GtkCharSelection *charsel, gint selection)
   if(selection >= 256) return;
 
   if(charsel->selection >= 0){
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(charsel->button[charsel->selection]), FALSE);
+      GTK_BUTTON(charsel->button[charsel->selection])->button_down=FALSE;
+      GTK_TOGGLE_BUTTON(charsel->button[charsel->selection])->active=FALSE;
+      gtk_widget_set_state(GTK_WIDGET(charsel->button[charsel->selection]), GTK_STATE_NORMAL);
+
       if(GTK_WIDGET_MAPPED(GTK_WIDGET(charsel)))
          gtk_widget_queue_draw(GTK_WIDGET(charsel->button[charsel->selection]));
   }
@@ -319,7 +301,9 @@ gtk_char_selection_set_selection(GtkCharSelection *charsel, gint selection)
   charsel->selection = selection;
 
   if(charsel->selection >= 0){
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(charsel->button[selection]), TRUE);
+      GTK_BUTTON(charsel->button[selection])->button_down=TRUE;
+      GTK_TOGGLE_BUTTON(charsel->button[selection])->active=TRUE;
+      gtk_widget_set_state(GTK_WIDGET(charsel->button[selection]), GTK_STATE_ACTIVE);
 
       if(GTK_WIDGET_MAPPED(GTK_WIDGET(charsel)))
          gtk_widget_queue_draw(GTK_WIDGET(charsel->button[selection]));
