@@ -187,7 +187,6 @@ gtk_file_list_construct(GtkFileList *file_list,
 
   icon_list->mode = mode;
   icon_list->icon_width = icon_width;
-  icon_list->selection_mode = GTK_SELECTION_SINGLE;
 
   if(path)
      file_list->path = g_strdup(path);
@@ -226,8 +225,7 @@ gtk_file_list_destroy(GtkObject *object)
   while(list){
     item = (GtkIconListItem *)list->data;
     file_item = (GtkFileListItem *)item->link;
-    if(file_item->file_name) g_free(file_item->file_name);
-    file_item->file_name = NULL;
+    g_free(file_item->file_name);
     g_free(item->link);
     item->link = NULL;
     list = list->next;
@@ -237,33 +235,22 @@ gtk_file_list_destroy(GtkObject *object)
   while(list){
     GtkFileListType *type;
     type = (GtkFileListType *)list->data;
-    if(type->extension) g_free(type->extension);
-    type->extension = NULL;
+    g_free(type->extension);
     file_list->types = g_list_remove_link(file_list->types, list);
     g_list_free_1(list);
     list = file_list->types;
   }
-  file_list->types = NULL;
 
   list = file_list->pixmaps;
   while(list){
     file_list->pixmaps = g_list_remove_link(file_list->pixmaps, list);
-    gtk_widget_destroy(GTK_WIDGET(list->data));
     g_list_free_1(list);
     list = file_list->pixmaps;
   }
-  file_list->pixmaps = NULL;
 
 
   g_free(GTK_FILE_LIST(object)->path); 
-  GTK_FILE_LIST(object)->path = NULL; 
-
   g_free(GTK_FILE_LIST(object)->filter); 
-  GTK_FILE_LIST(object)->filter = NULL; 
-
-  if (GTK_OBJECT_CLASS (parent_class)->destroy)
-  (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-
 }
 
 static void
@@ -383,7 +370,7 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
   gchar *real_path;
   GdkPixmap *pixmap;
   GdkBitmap *mask;
-  GtkPixmap *icon;
+  gchar **icon;
   gint width, height;
   GdkGC *gc;
   GList *types, *files, *list;
@@ -415,7 +402,7 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
            g_free(aux_path);
            aux_path = g_strdup(root);
         } else {
-           aux_path[length - 2] = '\0';
+           aux_path[length - 1] = '\0';
         }
      } else if(strcmp(aux_path + length - 3, root2) == 0){
         if(length == 3) {
@@ -425,7 +412,7 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
            gint i = length - 4;
            while(i >= 0){
               if(aux_path[i] == root[0]){ 
-                   aux_path[i] = '\0';
+                   aux_path[i+1] = '\0';
                    break;
               }
               i--;
@@ -439,7 +426,7 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
            gint i = length - 5;
            while(i >= 0){
               if(aux_path[i] == root[0]){ 
-                   aux_path[i] = '\0';
+                   aux_path[i+1] = '\0';
                    break;
               }
               i--;
@@ -450,17 +437,9 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
            g_free(aux_path);
            aux_path = g_strdup(root);
         } else {
-           aux_path[length - 3] = '\0';
+           aux_path[length - 2] = '\0';
         }
-     } else if(strcmp(aux_path + length - 1, root) == 0){
-       aux_path[length - 1] = '\0';
-     }
-
-     if(strlen(aux_path) == 0)
-     {
-       g_free(aux_path);
-       aux_path = g_strdup(G_DIR_SEPARATOR_S);
-     }
+     } 
   }
   else
      aux_path = g_strdup(G_DIR_SEPARATOR_S);
@@ -508,7 +487,7 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
 
   files = NULL;
   while((dirent=readdir(dir))!=NULL){
-        full_name = g_strconcat(real_path,G_DIR_SEPARATOR_S,dirent->d_name,NULL);
+        full_name = g_strconcat(real_path,dirent->d_name,NULL);
         file = dirent->d_name;
         show_file = TRUE;
         if(stat_subdirs){
@@ -591,15 +570,11 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
          types = types->next;
        }
 
-       icon = GTK_PIXMAP(g_list_nth_data(file_list->pixmaps, type));
+       icon = (gchar **)g_list_nth_data(file_list->pixmaps, type);
        file_item->type = type;
 
-       gdk_pixmap_ref(icon->pixmap);
-       gdk_bitmap_ref(icon->mask);
-       item = gtk_icon_list_add_from_pixmap(icon_list,
-                                            icon->pixmap,
-                                            icon->mask,
-                                            file_item->file_name, file_item);
+       item = gtk_icon_list_add_from_data(icon_list, icon, 
+                                          file_item->file_name, file_item);
 
        if(file_item->is_link){
                pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, 
@@ -626,7 +601,6 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
                gdk_gc_unref(gc);
   
                gdk_pixmap_unref(pixmap);
-               gdk_bitmap_unref(mask);
        }
 
        list = list->next;
@@ -649,13 +623,13 @@ gtk_file_list_set_filter(GtkFileList *filelist, const gchar *filter)
   gtk_file_list_open_dir(filelist, filelist->path);
 }
 
-const gchar *
+gchar *
 gtk_file_list_get_path(GtkFileList *file_list)
 {
   return(file_list->path);
 }
 
-const gchar *
+gchar *
 gtk_file_list_get_filename(GtkFileList *file_list)
 {
   GList *list;
@@ -694,32 +668,9 @@ gtk_file_list_get_filetype(GtkFileList *file_list)
 
 gint
 gtk_file_list_add_type       (GtkFileList *file_list,
-                              const gchar **data)
+                              const gchar **pixmap_data)
 {
-  GdkPixmap *pixmap;
-  GdkBitmap *mask;
-  gint type;
-
-  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
-                                                 gdk_colormap_get_system(),
-                                                 &mask, NULL,
-                                                 (gchar **)data);
-
-  type = gtk_file_list_add_type_with_pixmap(file_list, pixmap, mask);
-  gdk_pixmap_unref(pixmap);
-  gdk_bitmap_unref(mask);
-
-  return type;
-}
-
-gint
-gtk_file_list_add_type_with_pixmap (GtkFileList *file_list,
-                              GdkPixmap *pixmap, GdkBitmap *mask)
-{
-  GtkWidget *wpixmap;
-  wpixmap = gtk_pixmap_new(pixmap, mask);
-
-  file_list->pixmaps = g_list_append(file_list->pixmaps, wpixmap);
+  file_list->pixmaps = g_list_append(file_list->pixmaps, pixmap_data);
   file_list->ntypes++;
   return (file_list->ntypes - 1);
 }
