@@ -36,7 +36,6 @@
 #include <stdlib.h>
 
 #include "gtkiconfilesel.h"
-#include "gtkextraicons.h"
 
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 64
@@ -48,38 +47,37 @@
 
 static void gtk_icon_file_selection_class_init          (GtkIconFileSelClass *klass);
 static void gtk_icon_file_selection_init                (GtkIconFileSel *filesel);
-static void gtk_icon_file_selection_destroy             (GtkObject *object);
 static void open_dir					(GtkWidget *widget, 
 							 GtkCTreeNode *node, 
 							 gint n,
                                                          gpointer data);
-static gboolean entry_set_file				(GtkWidget *widget, 
+static void entry_set_file				(GtkWidget *widget, 
 							 GdkEventKey *key, 
 							 gpointer data);
 static void real_set_file				(GtkWidget *widget, 
 							 gpointer data);
-static gboolean set_filter				(GtkWidget *widget, 
+static void set_filter					(GtkWidget *widget, 
                                                          GdkEventKey *key,
 							 gpointer data);
 static gboolean select_icon				(GtkIconList *iconlist, 
             						 GtkIconListItem *icon,
             						 GdkEvent *event, 
 							 gpointer data);
-static gboolean insert_text                             (GtkEditable *editable, 
-                                                         const gchar *new_text,
-                                                         gint  new_text_length, 
-                                                         gint  *position,
-                                                         gpointer data);
+static void insert_text     				(GtkEditable *editable,
+                 					 const gchar *new_text,
+                 					 gint  new_text_length,
+                 					 gint  *position,
+                 					 gpointer data);
 static void init_history_combo				(GtkIconFileSel *filesel, 
 							 const gchar *curr_dir);
 static void update_history_combo			(GtkIconFileSel *filesel, 
 							 const gchar *curr_dir);
 static void go_to_history				(GtkEntry *entry,
 							 gpointer data);
-static gboolean combo_changed				(GtkWidget *widget, 
+static void combo_changed				(GtkWidget *widget, 
 							 GtkWidget *child,
 							 gpointer data);
-static gboolean entry_key_press				(GtkWidget *widget, 
+static void entry_key_press				(GtkWidget *widget, 
 							 GdkEventKey *event, 
 							 gpointer data);
 static gchar *get_real_path				(const gchar *full_path);
@@ -136,55 +134,12 @@ static void
 gtk_icon_file_selection_class_init (GtkIconFileSelClass *klass)
 {
   GtkWidgetClass *widget_class;
-  GtkObjectClass *object_class;
-
+  
   widget_class = (GtkWidgetClass*) klass;
-  object_class = (GtkObjectClass*) klass;
   parent_class = gtk_type_class (gtk_window_get_type ());
 
-  object_class->destroy = gtk_icon_file_selection_destroy;
 }
 
-static void
-up_clicked(GtkWidget *widget, gpointer data)
-{
-  GtkIconFileSel *filesel = GTK_ICON_FILESEL(widget);
-  gchar *current_dir;
-  gint dir_len;
-  gint i;
-
-  current_dir = g_strdup (GTK_FILE_LIST(filesel->file_list)->path);
-  dir_len = strlen (current_dir);
-
-  for (i = dir_len - 1; i >= 0; i--){
-
-    /* the i == dir_len is to catch the full path for the first
-     * entry. */
-    if ( current_dir[i] == G_DIR_SEPARATOR ) {
-          current_dir[i + 1] = '\0';
-          gtk_icon_file_selection_open_dir(filesel, current_dir);
-          break;
-     }
-  }
-
-  g_free(current_dir);
-}
-
-static void
-refresh_clicked(GtkWidget *widget, gpointer data)
-{
-  GtkIconFileSel *filesel = GTK_ICON_FILESEL(widget);
-  gtk_icon_file_selection_open_dir(filesel, GTK_FILE_LIST(filesel->file_list)->path);
-}
-
-static void
-home_clicked(GtkWidget *widget, gpointer data)
-{
-  GtkIconFileSel *filesel = GTK_ICON_FILESEL(widget);
-  gtk_icon_file_selection_open_dir(filesel, g_get_home_dir());
-}
-
-
 static void
 gtk_icon_file_selection_init (GtkIconFileSel *filesel)
 {
@@ -193,13 +148,8 @@ gtk_icon_file_selection_init (GtkIconFileSel *filesel)
   GtkWidget *table;
   GtkWidget *label;
   GtkWidget *scrolled_window;
-  GtkWidget *wpixmap;
-  GtkTooltips *tp;
-  GdkPixmap *pixmap;
-  GdkBitmap *mask;
   gchar cwd_path[2*MAXPATHLEN] = "";
   gchar path[2*MAXPATHLEN] = "";
-  GdkColormap *colormap = gtk_widget_get_colormap(GTK_WIDGET(filesel));
 
   filesel->show_tree = FALSE;
 
@@ -240,54 +190,6 @@ gtk_icon_file_selection_init (GtkIconFileSel *filesel)
 		     "select_child",
                      GTK_SIGNAL_FUNC(combo_changed), filesel);
 
-  filesel->up_button = gtk_button_new();
-  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
-                                                 up_xpm);
-  wpixmap = gtk_pixmap_new(pixmap, mask);
-  gdk_pixmap_unref(pixmap);
-  gdk_bitmap_unref(mask);
-
-  gtk_container_add(GTK_CONTAINER(filesel->up_button), wpixmap);
-  gtk_box_pack_start(GTK_BOX(hbox), filesel->up_button, FALSE, FALSE, 0);
-  gtk_widget_show_all(filesel->up_button);
-  gtk_signal_connect_object(GTK_OBJECT(filesel->up_button), "clicked",
-		            GTK_SIGNAL_FUNC(up_clicked), GTK_OBJECT(filesel));
-  tp = gtk_tooltips_new();
-  gtk_tooltips_set_tip(GTK_TOOLTIPS(tp), filesel->up_button, "Parent directory", "Parent directory");
-  gtk_tooltips_enable(GTK_TOOLTIPS(tp));
-
-  filesel->home_button = gtk_button_new();
-  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
-                                                 home_xpm);
-  wpixmap = gtk_pixmap_new(pixmap, mask);
-  gdk_pixmap_unref(pixmap);
-  gdk_bitmap_unref(mask);
-
-  gtk_container_add(GTK_CONTAINER(filesel->home_button), wpixmap);
-  gtk_box_pack_start(GTK_BOX(hbox), filesel->home_button, FALSE, FALSE, 0);
-  gtk_widget_show_all(filesel->home_button);
-  gtk_signal_connect_object(GTK_OBJECT(filesel->home_button), "clicked",
-		            GTK_SIGNAL_FUNC(home_clicked), GTK_OBJECT(filesel));
-  tp = gtk_tooltips_new();
-  gtk_tooltips_set_tip(GTK_TOOLTIPS(tp), filesel->home_button, "Home", "Home");
-  gtk_tooltips_enable(GTK_TOOLTIPS(tp));
-
-  filesel->refresh_button = gtk_button_new();
-  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
-                                                 refresh_xpm);
-  wpixmap = gtk_pixmap_new(pixmap, mask);
-  gdk_pixmap_unref(pixmap);
-  gdk_bitmap_unref(mask);
-
-  gtk_container_add(GTK_CONTAINER(filesel->refresh_button), wpixmap);
-  gtk_box_pack_start(GTK_BOX(hbox), filesel->refresh_button, FALSE, FALSE, 0);
-  gtk_widget_show_all(filesel->refresh_button);
-  gtk_signal_connect_object(GTK_OBJECT(filesel->refresh_button), "clicked",
-		            GTK_SIGNAL_FUNC(refresh_clicked), GTK_OBJECT(filesel));
-  tp = gtk_tooltips_new();
-  gtk_tooltips_set_tip(GTK_TOOLTIPS(tp), filesel->refresh_button, "Refresh", "Refresh");
-  gtk_tooltips_enable(GTK_TOOLTIPS(tp));
-
   filesel->path_label = gtk_label_new(path);
   gtk_misc_set_alignment(GTK_MISC(filesel->path_label), 0., .5);
   gtk_box_pack_start(GTK_BOX(main_vbox), filesel->path_label, FALSE, TRUE, 0);
@@ -304,14 +206,13 @@ gtk_icon_file_selection_init (GtkIconFileSel *filesel)
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
 
-/* FIXME
   filesel->dir_tree = gtk_dir_tree_new();
   GTK_DIR_TREE(filesel->dir_tree)->show_hidden = TRUE;
   gtk_container_add(GTK_CONTAINER(scrolled_window), filesel->dir_tree);
   gtk_widget_show(filesel->dir_tree);
 
+
   gtk_box_pack_start(GTK_BOX(hbox), gtk_vseparator_new(), TRUE, TRUE, 0);
-*/
 
   filesel->list_window = scrolled_window=gtk_scrolled_window_new(NULL, NULL);
   gtk_box_pack_start(GTK_BOX(hbox), scrolled_window, TRUE, TRUE, 0);
@@ -371,64 +272,36 @@ gtk_icon_file_selection_init (GtkIconFileSel *filesel)
   gtk_signal_connect(GTK_OBJECT(filesel->filter_entry), "key_press_event",
                      GTK_SIGNAL_FUNC(set_filter), filesel);
 
-/* FIXME
   gtk_signal_connect(GTK_OBJECT(filesel->filter_entry), "insert_text",
                      GTK_SIGNAL_FUNC(insert_text), NULL);
-*/
 
   box = gtk_vbutton_box_new();
   gtk_table_attach_defaults(GTK_TABLE(table), box, 3, 4, 0, 2);
   gtk_widget_show(box);
 
-  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
-                                                 ok_xpm);
-  wpixmap = gtk_pixmap_new(pixmap, mask);
-  gdk_pixmap_unref(pixmap);
-  gdk_bitmap_unref(mask);
-
-  filesel->ok_button = gtk_button_new_from_stock(GTK_STOCK_OK);
+  filesel->ok_button = gtk_button_new_with_label("OK");
   gtk_box_pack_end (GTK_BOX (box), filesel->ok_button, TRUE, TRUE, 0);
   gtk_widget_show(filesel->ok_button);
 
   gtk_signal_connect(GTK_OBJECT(filesel->ok_button), "clicked",
                      GTK_SIGNAL_FUNC(real_set_file), filesel);
 
-  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
-                                                 cancel_xpm);
-  wpixmap = gtk_pixmap_new(pixmap, mask);
-  gdk_pixmap_unref(pixmap);
-  gdk_bitmap_unref(mask);
-
-  filesel->cancel_button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+  filesel->cancel_button = gtk_button_new_with_label("Cancel");
   gtk_box_pack_end (GTK_BOX (box), filesel->cancel_button, TRUE, TRUE, 0);
   gtk_widget_show(filesel->cancel_button);
 
   gtk_icon_file_selection_open_dir(filesel, path);
-
-  filesel->selection = NULL;
-}
-
-static void
-gtk_icon_file_selection_destroy(GtkObject *object)
-{
-  if(GTK_ICON_FILESEL(object)->selection)
-    g_free(GTK_ICON_FILESEL(object)->selection);
-  GTK_ICON_FILESEL(object)->selection = NULL;
-
-  if (GTK_OBJECT_CLASS (parent_class)->destroy)
-    (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
 void
 gtk_icon_file_selection_show_tree(GtkIconFileSel *filesel, gboolean show)
 {
-/* FIXME
   if(show == filesel->show_tree) return;
 
   filesel->show_tree = show;
 
   if(show){
-    const gchar *path;
+    gchar *path;
 
     filesel->tree_signal_id = gtk_signal_connect(GTK_OBJECT(filesel->dir_tree), 
                                                  "tree_select_row",
@@ -446,28 +319,25 @@ gtk_icon_file_selection_show_tree(GtkIconFileSel *filesel, gboolean show)
     gtk_widget_hide(filesel->tree_window);
     gtk_widget_set_usize(filesel->list_window, 550, 250);
   } 
-*/
 }
 
-static gboolean
+static void
 insert_text     (GtkEditable *editable,
                  const gchar *new_text,
                  gint         new_text_length,
                  gint        *position,
                  gpointer data)
 {
-  g_object_ref(editable);
   gtk_signal_emit_stop_by_name(GTK_OBJECT(editable), "insert_text");
   if(new_text[0] != ' '){
-     GTK_EDITABLE_CLASS (gtk_type_class(GTK_TYPE_EDITABLE))->insert_text(editable,
+     GTK_EDITABLE_CLASS (gtk_type_class(GTK_TYPE_ENTRY))->insert_text(editable,
                                                               new_text,
-                                                              new_text_length,
+                                                              new_text_length, 
                                                               position);
-                                                                               
-                                                                               
+
+
   }
-  g_object_unref(editable);
-  return TRUE;
+
 }
 
 static gboolean 
@@ -477,10 +347,10 @@ select_icon(GtkIconList *iconlist,
 {
   GtkIconFileSel *filesel;
   GdkModifierType mods;
-  const gchar *path = NULL;
+  gchar *path = NULL;
   gchar *real_path = NULL;
   gchar *full_path = NULL;
-  const gchar *file = NULL;
+  gchar *file = NULL;
   GtkFileListItem *item;
   gboolean return_val = FALSE;
 
@@ -489,20 +359,7 @@ select_icon(GtkIconList *iconlist,
   filesel = GTK_ICON_FILESEL(data);
 
   if(item->type != GTK_FILE_LIST_FOLDER){
-    GList *list = iconlist->selection;
-    if(iconlist->selection_mode == GTK_SELECTION_MULTIPLE && list != NULL){
-      gchar *text = g_strdup(((GtkIconListItem *)list->data)->label); 
-      list = list->next;
-      while(list){
-        text = g_strconcat(text,";",((GtkIconListItem *)list->data)->label,NULL);
-        list = list->next;
-      }
-      text = g_strconcat(text,";",icon->label,NULL);
-      gtk_entry_set_text(GTK_ENTRY(filesel->file_entry), text);
-      g_free(text);
-    } else {
-      gtk_entry_set_text(GTK_ENTRY(filesel->file_entry), icon->label);
-    }
+    gtk_entry_set_text(GTK_ENTRY(filesel->file_entry), icon->label);
     return TRUE;
   }else{
     gtk_entry_set_text(GTK_ENTRY(filesel->file_entry), "");
@@ -515,53 +372,42 @@ select_icon(GtkIconList *iconlist,
   else
     return FALSE; 
 
-  path = gtk_file_list_get_path(GTK_FILE_LIST(filesel->file_list));
-  file = gtk_file_list_get_filename(GTK_FILE_LIST(filesel->file_list));
-  file = icon->label;
-
-  if(strlen(path) == 1)
-    full_path = g_strconcat(G_DIR_SEPARATOR_S,file,G_DIR_SEPARATOR_S,NULL);
-  else
-    full_path = g_strconcat(path,G_DIR_SEPARATOR_S,file,G_DIR_SEPARATOR_S,NULL);
-  real_path = get_real_path((const gchar *)full_path);
-
-  if(filesel->selection) g_free(filesel->selection);
-  filesel->selection = NULL;
-  if(item->type != GTK_FILE_LIST_FOLDER){
-    filesel->selection = g_strdup(real_path);
-  }
-
   if((mods & GDK_BUTTON1_MASK) && event->type == GDK_2BUTTON_PRESS){
+    path = gtk_file_list_get_path(GTK_FILE_LIST(filesel->file_list));
+    file = gtk_file_list_get_filename(GTK_FILE_LIST(filesel->file_list));
+    file = icon->label;
+
+    full_path = (gchar *)g_malloc(strlen(path)+strlen(file)+3);
+    g_snprintf(full_path,strlen(path)+strlen(file)+2,"%s%s%s",path,file,G_DIR_SEPARATOR_S);
+    real_path = get_real_path((const gchar *)full_path);
+
     gtk_label_set_text(GTK_LABEL(filesel->path_label), "Scanning...");
-    if(filesel->show_tree){
-/* FIXME
+    if(filesel->show_tree)
       return_val = gtk_dir_tree_open_dir(GTK_DIR_TREE(filesel->dir_tree), real_path);
-*/
-    } else
+    else
       return_val = gtk_file_list_open_dir(GTK_FILE_LIST(filesel->file_list), real_path);
 
     update_history_combo(filesel, real_path);
 
     gtk_label_set_text(GTK_LABEL(filesel->path_label), real_path);
+    g_free(full_path);
+    g_free(real_path);
+    return (!return_val);
   }
 
-  g_free(full_path);
-  g_free(real_path);
-  return (!return_val);
+  return TRUE;
 }
 
-static gboolean 
+static void
 entry_set_file(GtkWidget *widget, GdkEventKey *key, gpointer data)
 {
   GtkIconFileSel *filesel = GTK_ICON_FILESEL(data);
 
-  if(key->keyval != GDK_Return && key->keyval != GDK_KP_Enter) return FALSE;
+  if(key->keyval != GDK_Return && key->keyval != GDK_KP_Enter) return;
 
 /*  real_set_file(widget, data);
 */
   gtk_signal_emit_by_name(GTK_OBJECT(filesel->ok_button), "clicked", filesel);
-
-  return FALSE;
 }
 
 static void
@@ -570,8 +416,7 @@ real_set_file(GtkWidget *widget, gpointer data)
   GtkIconFileSel *filesel;
   GtkIconListItem *item;
   GList *list;
-  const gchar *c;
-  gchar *last, *text;
+  gchar *c, *last, *text;
   gchar *folder;
   gchar *file;
   gint nlen, file_len;
@@ -611,10 +456,7 @@ real_set_file(GtkWidget *widget, gpointer data)
       item = (GtkIconListItem *)list->data;
       text = ((GtkFileListItem *)item->link)->file_name;
       if(strcmp(text, file) == 0){
-         item->state = GTK_STATE_SELECTED;
-/*
          gtk_icon_list_select_icon(GTK_ICON_LIST(filesel->file_list), item);
-*/
          break;
       }
       list = list->next;
@@ -627,18 +469,16 @@ real_set_file(GtkWidget *widget, gpointer data)
   g_free(last);
 }
 
-static gboolean 
+static void
 set_filter(GtkWidget *widget, GdkEventKey *key, gpointer data)
 {
   GtkIconFileSel *filesel;
 
-  if(key->keyval != GDK_Return && key->keyval != GDK_KP_Enter) return FALSE;
+  if(key->keyval != GDK_Return && key->keyval != GDK_KP_Enter) return;
 
   filesel = (GtkIconFileSel *)data;
   gtk_file_list_set_filter(GTK_FILE_LIST(filesel->file_list), 
                            gtk_entry_get_text(GTK_ENTRY(widget)));
-
-  return TRUE;
 }
 
 static void
@@ -646,8 +486,7 @@ open_dir(GtkWidget *widget, GtkCTreeNode *node, gint n, gpointer data)
 {
   DIR *dir;
   GtkDirTreeNode *dirnode;
-  gchar *path;
-  const gchar *last_path;
+  gchar *path, *last_path;
   GtkIconFileSel *filesel;
 
   filesel = GTK_ICON_FILESEL(data);
@@ -683,7 +522,7 @@ gint
 gtk_icon_file_selection_open_dir(GtkIconFileSel *filesel, const gchar *path)
 {
   DIR *dir;
-  gint return_val = TRUE;
+  gint return_val;
   gchar *real_path = NULL;
  
   if(!path) return FALSE;
@@ -697,11 +536,9 @@ gtk_icon_file_selection_open_dir(GtkIconFileSel *filesel, const gchar *path)
 
   gtk_label_set_text(GTK_LABEL(filesel->path_label), "Scanning...");
 
-  if(filesel->show_tree){
-/* FIXME
+  if(filesel->show_tree)
     return_val = gtk_dir_tree_open_dir(GTK_DIR_TREE(filesel->dir_tree), real_path);
-*/
-  } else
+  else
     return_val = gtk_file_list_open_dir(GTK_FILE_LIST(filesel->file_list), real_path);
 
   gtk_label_set_text(GTK_LABEL(filesel->path_label), real_path);
@@ -716,9 +553,7 @@ gtk_icon_file_selection_open_dir(GtkIconFileSel *filesel, const gchar *path)
 void
 gtk_icon_file_selection_show_hidden(GtkIconFileSel *filesel, gboolean visible)
 {
-/* FIXME
     GTK_DIR_TREE(filesel->dir_tree)->show_hidden = visible;
-*/
     GTK_FILE_LIST(filesel->file_list)->show_hidden = visible;
 }
 
@@ -755,7 +590,7 @@ get_real_path(const gchar *full_path)
         g_free(aux_path);
         aux_path = g_strdup(root);
      } else {
-        aux_path[length - 2] = '\0';
+        aux_path[length - 1] = '\0';
      }
   } else if(strcmp(aux_path + length - 3, root2) == 0){
      if(length == 3) {
@@ -765,7 +600,7 @@ get_real_path(const gchar *full_path)
         gint i = length - 4;
         while(i >= 0){
            if(aux_path[i] == root[0]){
-                aux_path[i] = '\0';
+                aux_path[i+1] = '\0';
                 break;
            }
            i--;
@@ -779,7 +614,7 @@ get_real_path(const gchar *full_path)
         gint i = length - 5;
         while(i >= 0){
            if(aux_path[i] == root[0]){
-                aux_path[i] = '\0';
+                aux_path[i+1] = '\0';
                 break;
            }
            i--;
@@ -790,17 +625,11 @@ get_real_path(const gchar *full_path)
         g_free(aux_path);
         aux_path = g_strdup(root);
      } else {
-        aux_path[length - 3] = '\0';
+        aux_path[length - 2] = '\0';
      }
-  } else if(strcmp(aux_path + length - 1, root) == 0 && length > 1){
-     aux_path[length - 1] = '\0';
-  }
-
-  if(strlen(aux_path) == 0)
-  {
-    g_free(aux_path);
-    aux_path = g_strdup(G_DIR_SEPARATOR_S);
   }
+  else
+     aux_path = g_strdup(full_path);
 
   return(aux_path);
 }
@@ -873,7 +702,7 @@ update_history_combo(GtkIconFileSel *filesel, const gchar *current_directory)
 static void
 go_to_history(GtkEntry *entry, gpointer data)
 {
-  const gchar *path;
+  gchar *path;
   gchar *real_path;
 
   path = gtk_entry_get_text(entry);
@@ -885,7 +714,7 @@ go_to_history(GtkEntry *entry, gpointer data)
   g_free(real_path);
 }
 
-static gboolean 
+static void
 combo_changed(GtkWidget *widget, GtkWidget *child, gpointer data)
 {
   GtkCombo *combo;
@@ -899,12 +728,10 @@ combo_changed(GtkWidget *widget, GtkWidget *child, gpointer data)
   gtk_signal_handler_block (GTK_OBJECT (combo->list), combo->list_change_id);
   go_to_history(entry, filesel);
   gtk_signal_handler_unblock (GTK_OBJECT (combo->list), combo->list_change_id);
-
-  return TRUE;
 }
 
 
-static gboolean 
+static void
 entry_key_press(GtkWidget *widget, 
 	        GdkEventKey *event, 
 		gpointer data)
@@ -914,13 +741,6 @@ entry_key_press(GtkWidget *widget,
   if(event->keyval == GDK_Return){
      gtk_signal_emit_stop_by_name(GTK_OBJECT(entry), "key_press_event");
      go_to_history(entry, data);
-     return TRUE;
   }
-  return FALSE;
 } 
 
-const gchar *
-gtk_icon_file_selection_get_selection(GtkIconFileSel *filesel)
-{
-  return filesel->selection;
-}
