@@ -30,15 +30,8 @@
 #include "gtkplotps.h"
 #include "gtkpsfont.h"
 
-enum {
-  ARG_0,
-  ARG_PIXMAP,
-  ARG_MASK,
-};
-                                                                                
 static void gtk_plot_pixmap_class_init 		(GtkPlotPixmapClass *klass);
 static void gtk_plot_pixmap_init 		(GtkPlotPixmap *data);
-static void gtk_plot_pixmap_destroy             (GtkObject *object);
 static void gtk_plot_pixmap_draw_symbol		(GtkPlotData *data,
                                                  gdouble x, 
                                                  gdouble y, 
@@ -52,19 +45,8 @@ static void gtk_plot_pixmap_draw_legend		(GtkPlotData *data,
 					 	 gint x, gint y);
 static void gtk_plot_pixmap_get_legend_size	(GtkPlotData *data, 
 						 gint *width, gint *height);
-static void gtk_plot_pixmap_clone               (GtkPlotData *data,
-                                                 GtkPlotData *copy);
-static void gtk_plot_pixmap_get_arg      	(GtkObject      *object,
-                                                 GtkArg         *arg,
-                                                 guint           arg_id);
-static void gtk_plot_pixmap_set_arg      	(GtkObject      *object,
-                                                 GtkArg         *arg,
-                                                 guint           arg_id);
-                                                                                
 
-
-
-extern inline gint roundint 			(gdouble x);
+static gint roundint 				(gdouble x);
 
 static GtkPlotDataClass *parent_class = NULL;
 
@@ -105,65 +87,11 @@ gtk_plot_pixmap_class_init (GtkPlotPixmapClass *klass)
   widget_class = (GtkWidgetClass *) klass;
   data_class = (GtkPlotDataClass *) klass;
 
-  data_class->clone = gtk_plot_pixmap_clone;
   data_class->draw_legend = gtk_plot_pixmap_draw_legend;
   data_class->get_legend_size = gtk_plot_pixmap_get_legend_size;
   data_class->draw_symbol = gtk_plot_pixmap_draw_symbol;
-
-  object_class->destroy = gtk_plot_pixmap_destroy;
-
-  object_class->get_arg = gtk_plot_pixmap_get_arg;
-  object_class->set_arg = gtk_plot_pixmap_set_arg;
-                                                                                
-  gtk_object_add_arg_type ("GtkPlotPixmap::pixmap",
-                           GTK_TYPE_POINTER,
-                           GTK_ARG_READWRITE,
-                           ARG_PIXMAP);
-  gtk_object_add_arg_type ("GtkPlotPixmap::mask_bitmap",
-                           GTK_TYPE_POINTER,
-                           GTK_ARG_READWRITE,
-                           ARG_MASK);
-
 }
 
-static void
-gtk_plot_pixmap_get_arg (GtkObject      *object,
-                         GtkArg         *arg,
-                         guint           arg_id)
-{
-  GtkPlotPixmap *pixmap = GTK_PLOT_PIXMAP (object);
-                                                                                
-  switch(arg_id){
-    case ARG_PIXMAP:
-      GTK_VALUE_POINTER(*arg) = &pixmap->pixmap;
-      break;
-    case ARG_MASK:
-      GTK_VALUE_POINTER(*arg) = &pixmap->mask;
-      break;
-  }
-}
-                                                                                
-static void
-gtk_plot_pixmap_set_arg (GtkObject      *object,
-                         GtkArg         *arg,
-                         guint           arg_id)
-{
-  GtkPlotPixmap *pixmap = GTK_PLOT_PIXMAP (object);
-                                                                                
-  switch(arg_id){
-    case ARG_PIXMAP:
-      if(pixmap->pixmap) gdk_pixmap_unref(pixmap->pixmap);
-      pixmap->pixmap = (GdkPixmap *)GTK_VALUE_POINTER(*arg);
-      if(pixmap->pixmap) gdk_pixmap_ref(pixmap->pixmap);
-      break;
-    case ARG_MASK:
-      if(pixmap->mask) gdk_bitmap_unref(pixmap->mask);
-      pixmap->mask = (GdkBitmap *)GTK_VALUE_POINTER(*arg);
-      if(pixmap->mask) gdk_bitmap_ref(pixmap->mask);
-      break;
-  }
-}
-                                                                                
 
 static void
 gtk_plot_pixmap_init (GtkPlotPixmap *dataset)
@@ -195,28 +123,6 @@ gtk_plot_pixmap_construct(GtkPlotPixmap *data, GdkPixmap *pixmap, GdkBitmap *mas
     gdk_bitmap_ref(mask);
 }
 
-static void
-gtk_plot_pixmap_destroy(GtkObject *object)
-{
-  GtkPlotPixmap *pixmap = GTK_PLOT_PIXMAP(object);
-                                                                                
-  if(pixmap->pixmap) gdk_pixmap_unref(pixmap->pixmap);
-  if(pixmap->mask) gdk_bitmap_unref(pixmap->mask);
-  pixmap->pixmap = NULL;
-  pixmap->mask = NULL;
-}
-
-static void
-gtk_plot_pixmap_clone(GtkPlotData *data, GtkPlotData *copy)
-{
-  GTK_PLOT_DATA_CLASS(parent_class)->clone(data, copy);
-
-  GTK_PLOT_PIXMAP(copy)->pixmap = GTK_PLOT_PIXMAP(data)->pixmap;
-  gdk_pixmap_ref(GTK_PLOT_PIXMAP(data)->pixmap);
-  GTK_PLOT_PIXMAP(copy)->mask = GTK_PLOT_PIXMAP(data)->mask;
-  gdk_bitmap_ref(GTK_PLOT_PIXMAP(data)->mask);
-}
-
 static void
 gtk_plot_pixmap_draw_symbol(GtkPlotData *data,
                          gdouble x, gdouble y, gdouble z, gdouble a,
@@ -280,6 +186,7 @@ gtk_plot_pixmap_draw_legend(GtkPlotData *data, gint x, gint y)
 
   g_return_if_fail(data->plot != NULL);
   g_return_if_fail(GTK_IS_PLOT(data->plot));
+  g_return_if_fail(GTK_WIDGET_REALIZED(data->plot));
 
   pixmap = GTK_PLOT_PIXMAP(data);
 
@@ -388,4 +295,13 @@ gtk_plot_pixmap_get_mask (GtkPlotPixmap *pixmap)
   return(pixmap->mask);
 }
 
+static gint
+roundint (gdouble x)
+{
+ gint sign = 1;
+
+/* if(x <= 0.) sign = -1;
+*/
+ return (x+sign*.50999999471);
+}
 
