@@ -34,15 +34,11 @@
 
 static void gtk_plot_polar_class_init 		(GtkPlotPolarClass *klass);
 static void gtk_plot_polar_init 		(GtkPlotPolar *plot);
-static void gtk_plot_polar_set_arg              (GtkObject *object,
-                                                 GtkArg    *arg,
-                                                 guint      arg_id);
-static void gtk_plot_polar_get_arg              (GtkObject *object,
-                                                 GtkArg    *arg,
-                                                 guint      arg_id);
 static void gtk_plot_polar_draw 		(GtkWidget *widget, 
 						 GdkRectangle *area);
 static void gtk_plot_polar_real_paint 		(GtkWidget *widget);
+static void gtk_plot_calc_ticks			(GtkPlotPolar *plot, 
+						 GtkPlotAxis *axis);
 static void gtk_plot_polar_draw_grids           (GtkPlotPolar *plot); 
 static void gtk_plot_polar_draw_axis		(GtkPlotPolar *plot, 
 					 	 GtkPlotAxis *axis,
@@ -61,15 +57,15 @@ static void gtk_plot_polar_real_get_point	(GtkWidget *widget,
 						 gint py,
                           			 gdouble *x, 
 						 gdouble *y); 
-extern inline gint roundint			(gdouble x);
+static gint roundint				(gdouble x);
+static void parse_label			        (gdouble val, 
+						 gint precision, 
+						 gint style,
+                                                 gchar *label);
+static gdouble transform			(GtkPlot *plot, gdouble x);
 
 static GtkPlotClass *parent_class = NULL;
 
-enum
-{
-  ARG_0,
-  ARG_ROTATION,
-};
 
 GtkType
 gtk_plot_polar_get_type (void)
@@ -108,19 +104,11 @@ gtk_plot_polar_class_init (GtkPlotPolarClass *klass)
   widget_class = (GtkWidgetClass *) klass;
   plot_class = (GtkPlotClass *) klass;
 
-  object_class->set_arg = gtk_plot_polar_set_arg;
-  object_class->get_arg = gtk_plot_polar_get_arg;
-
   widget_class->draw = gtk_plot_polar_draw;
 
   plot_class->plot_paint = gtk_plot_polar_real_paint;
   plot_class->get_point = gtk_plot_polar_real_get_point;
   plot_class->get_pixel = gtk_plot_polar_real_get_pixel;
-
-  gtk_object_add_arg_type ("GtkPlotPolar::rotation",
-                           GTK_TYPE_DOUBLE,
-                           GTK_ARG_READWRITE,
-                           ARG_ROTATION);
 }
 
 
@@ -135,25 +123,25 @@ gtk_plot_polar_init (GtkPlotPolar *plot)
   plot->r = GTK_PLOT(plot)->left;
   plot->angle = GTK_PLOT(plot)->bottom;
 
-  GTK_PLOT(plot)->bottom->ticks.min = 0.0;
-  GTK_PLOT(plot)->bottom->ticks.max = 360.0;
+  GTK_PLOT(plot)->bottom->min = 0.0;
+  GTK_PLOT(plot)->bottom->max = 360.0;
   GTK_PLOT(plot)->bottom->ticks.step = 30.;
-  GTK_PLOT(plot)->top->ticks.min = 0.0;
-  GTK_PLOT(plot)->top->ticks.max = 360.0;
+  GTK_PLOT(plot)->top->min = 0.0;
+  GTK_PLOT(plot)->top->max = 360.0;
   GTK_PLOT(plot)->top->ticks.step = 30.;
 
   GTK_PLOT(plot)->bottom->labels_offset = 15.;
   GTK_PLOT(plot)->bottom->label_precision = 0;
 
   GTK_PLOT(plot)->ymin = 0.2;
-  GTK_PLOT(plot)->left->ticks.min = 0.2;
+  GTK_PLOT(plot)->left->min = 0.2;
   GTK_PLOT(plot)->left->ticks.step = 0.2;
-  GTK_PLOT(plot)->right->ticks.min = 0.2;
+  GTK_PLOT(plot)->right->min = 0.2;
   GTK_PLOT(plot)->right->ticks.step = 0.2;
 
   GTK_PLOT(plot)->left->title.angle = 0.;
 
-  gtk_plot_axis_set_title(GTK_PLOT(plot)->left, "R");
+  gtk_plot_axis_set_title(GTK_PLOT(plot), GTK_PLOT_AXIS_LEFT, "R");
 
   GTK_PLOT(plot)->top->is_visible = FALSE;
 
@@ -165,41 +153,6 @@ gtk_plot_polar_init (GtkPlotPolar *plot)
   plot->rotation = 0.0;
 }
 
-static void
-gtk_plot_polar_get_arg (GtkObject      *object,
-                        GtkArg         *arg,
-                        guint           arg_id)
-{
-  GtkPlotPolar *plot;
-
-  plot = GTK_PLOT_POLAR (object);
-
-  switch(arg_id){
-    case ARG_ROTATION:
-      GTK_VALUE_DOUBLE(*arg) = plot->rotation;
-      break;
-    default:
-      arg->type = GTK_TYPE_INVALID;
-      break;
-  }
-}
-
-static void
-gtk_plot_polar_set_arg (GtkObject      *object,
-                        GtkArg         *arg,
-                        guint           arg_id)
-{
-  GtkPlotPolar *plot;
-
-  plot = GTK_PLOT_POLAR (object);
-
-  switch(arg_id){
-    case ARG_ROTATION:
-      plot->rotation = GTK_VALUE_DOUBLE(*arg);
-      break;
-  }
-}
-
 static void
 gtk_plot_polar_draw (GtkWidget *widget, GdkRectangle *area)
 {
@@ -216,18 +169,23 @@ gtk_plot_polar_real_paint (GtkWidget *widget)
   GdkPixmap *pixmap;
   GList *dataset;
   GList *text;
+  GdkRectangle area;
   gint width, height;
   gint xoffset, yoffset ;
   gdouble min;
 
-  if(!GTK_WIDGET_VISIBLE(widget)) return;
+  if(!GTK_WIDGET_REALIZED(widget)) return;
 
   plot = GTK_PLOT(widget);
 
-  xoffset = plot->internal_allocation.x;
-  yoffset = plot->internal_allocation.y;
-  width = plot->internal_allocation.width;
-  height = plot->internal_allocation.height;
+  area.x = widget->allocation.x;
+  area.y = widget->allocation.y;
+  area.width = widget->allocation.width;
+  area.height = widget->allocation.height;
+  xoffset = area.x + roundint(plot->x * widget->allocation.width);
+  yoffset = area.y + roundint(plot->y * widget->allocation.height);
+  width = roundint(plot->width * widget->allocation.width);
+  height = roundint(plot->height * widget->allocation.height);
 
   style = gtk_widget_get_style(widget);
 
@@ -249,11 +207,10 @@ gtk_plot_polar_real_paint (GtkWidget *widget)
 
   /* draw the ticks & grid lines */
 
-  min = plot->left->ticks.min;
-  plot->left->ticks.min = 0.0;
-  gtk_plot_axis_ticks_recalc(plot->left);
-  gtk_plot_axis_ticks_recalc(plot->bottom);
-  plot->left->ticks.min = min;
+  min = plot->left->min;
+  plot->left->min = 0.0;
+  gtk_plot_calc_ticks(GTK_PLOT_POLAR(plot), plot->left);
+  gtk_plot_calc_ticks(GTK_PLOT_POLAR(plot), plot->bottom);
 
   if(plot->left->is_visible)
     {
@@ -289,6 +246,8 @@ gtk_plot_polar_real_paint (GtkWidget *widget)
       gtk_plot_polar_draw_circle(GTK_PLOT_POLAR(plot));
     }
 
+  plot->left->min = min;
+  gtk_plot_calc_ticks(GTK_PLOT_POLAR(plot), plot->left);
   gtk_plot_polar_draw_grids(GTK_PLOT_POLAR(plot));
 
   dataset = plot->data_sets;
@@ -368,10 +327,10 @@ gtk_plot_polar_draw_grids(GtkPlotPolar *polar)
 
   rotation = polar->rotation;
 
-  xp = plot->internal_allocation.x;
-  yp = plot->internal_allocation.y;
-  width = plot->internal_allocation.width;
-  height = plot->internal_allocation.height;
+  xp = widget->allocation.x + plot->x * widget->allocation.width;
+  yp = widget->allocation.y + plot->y * widget->allocation.height;
+  width = plot->width * widget->allocation.width;
+  height = plot->height * widget->allocation.height;
 
   ox = xp + width / 2.;
   oy = yp + height / 2.;
@@ -379,16 +338,15 @@ gtk_plot_polar_draw_grids(GtkPlotPolar *polar)
 
   if(plot->bottom->show_minor_grid)
     {
-          for(ntick = 0; ntick < plot->bottom->ticks.nticks; ntick++){
-            if(!plot->bottom->ticks.values[ntick].minor) continue;
-            if(plot->bottom->ticks.values[ntick].value >= plot->bottom->ticks.min){
+          for(ntick = 0; ntick < plot->bottom->ticks.nminorticks; ntick++){
+            if(plot->bottom->ticks.minor_values[ntick] >= plot->bottom->min){
               gtk_plot_get_pixel(plot,
                                  plot->ymax,
-                                 plot->bottom->ticks.values[ntick].value,
+                                 plot->bottom->ticks.minor_values[ntick],
                                  &x1, &y1);
               gtk_plot_get_pixel(plot,
                                  plot->ymin,
-                                 plot->bottom->ticks.values[ntick].value,
+                                 plot->bottom->ticks.minor_values[ntick],
                                  &x2, &y2);
               gtk_plot_draw_line(plot, plot->bottom->minor_grid,
                                  x1, y1, x2, y2);
@@ -397,16 +355,15 @@ gtk_plot_polar_draw_grids(GtkPlotPolar *polar)
     }
   if(plot->bottom->show_major_grid)
     {
-          for(ntick = 0; ntick < plot->bottom->ticks.nticks; ntick++){
-           if(plot->bottom->ticks.values[ntick].minor) continue;
-           if(plot->bottom->ticks.values[ntick].value >= plot->bottom->ticks.min){
+          for(ntick = 0; ntick < plot->bottom->ticks.nmajorticks; ntick++){
+           if(plot->bottom->ticks.major_values[ntick] >= plot->bottom->min){
               gtk_plot_get_pixel(plot,
                                  plot->ymax,
-                                 plot->bottom->ticks.values[ntick].value,
+                                 plot->bottom->ticks.major_values[ntick],
                                  &x1, &y1);
               gtk_plot_get_pixel(plot,
                                  plot->ymin,
-                                 plot->bottom->ticks.values[ntick].value,
+                                 plot->bottom->ticks.major_values[ntick],
                                  &x2, &y2);
               gtk_plot_draw_line(plot, plot->bottom->major_grid,
                                  x1, y1, x2, y2);
@@ -416,11 +373,10 @@ gtk_plot_polar_draw_grids(GtkPlotPolar *polar)
   if(plot->left->show_minor_grid)
     {
           gtk_plot_set_line_attributes(plot, plot->left->minor_grid); 
-          for(ntick = 0; ntick < plot->left->ticks.nticks; ntick++){
-            if(!plot->left->ticks.values[ntick].minor) continue;
-            if(plot->left->ticks.values[ntick].value >= plot->left->ticks.min){
+          for(ntick = 0; ntick < plot->left->ticks.nminorticks; ntick++){
+            if(plot->left->ticks.minor_values[ntick] >= plot->left->min){
               gtk_plot_get_pixel(plot,
-                                 plot->left->ticks.values[ntick].value,
+                                 plot->left->ticks.minor_values[ntick],
                                  90.0 - rotation,
                                  &ix, &iy);
               iy = fabs(oy - iy);
@@ -435,11 +391,10 @@ gtk_plot_polar_draw_grids(GtkPlotPolar *polar)
 
     {
           gtk_plot_set_line_attributes(plot, plot->left->major_grid); 
-          for(ntick = 0; ntick < plot->left->ticks.nticks; ntick++){
-            if(plot->left->ticks.values[ntick].minor) continue;
-            if(plot->left->ticks.values[ntick].value >= plot->left->ticks.min){
+          for(ntick = 0; ntick < plot->left->ticks.nmajorticks; ntick++){
+            if(plot->left->ticks.major_values[ntick] >= plot->left->min){
               gtk_plot_get_pixel(plot,
-                                 plot->left->ticks.values[ntick].value,
+                                 plot->left->ticks.major_values[ntick],
                                  90.0 - rotation,
                                  &ix, &iy);
               iy = fabs(oy - iy);
@@ -460,24 +415,23 @@ gtk_plot_polar_draw_axis(GtkPlotPolar *polar,
   GtkWidget *widget;
   GtkPlot *plot;
   gdouble x, y;
-  gdouble xx, yy;
-  gdouble x_tick;
+  gdouble xx;
+  gint line_width;
   gdouble xp, yp, width, height, size;
   gint ntick;
   gdouble m;
   gdouble x1, y1;
   gdouble ox, oy;
-  gdouble x0, y0;
 
   widget = GTK_WIDGET(polar);
   plot = GTK_PLOT(polar);
 
   m = plot->magnification;
 
-  xp = plot->internal_allocation.x;
-  yp = plot->internal_allocation.y;
-  width = plot->internal_allocation.width;
-  height = plot->internal_allocation.height;
+  xp = widget->allocation.x + plot->x * widget->allocation.width;
+  yp = widget->allocation.y + plot->y * widget->allocation.height;
+  width = plot->width * widget->allocation.width;
+  height = plot->height * widget->allocation.height;
 
   size = MIN(width, height);
 
@@ -487,37 +441,30 @@ gtk_plot_polar_draw_axis(GtkPlotPolar *polar,
   x = xp + ox * axis->direction.x + axis->origin.x;
   y = yp + oy * axis->direction.y + axis->origin.y;
 
-  if(axis->direction.x == 0)
-    gtk_plot_get_pixel(plot, 0, -polar->rotation+90, &x0, &y0);
-  else
-    gtk_plot_get_pixel(plot, 0, -polar->rotation, &y0, &x0);
+  line_width = axis->line.line_width;
+  gtk_plot_pc_set_color(plot->pc, &axis->line.color);
 
-  gtk_plot_draw_line(plot, axis->line, 
+  gtk_plot_pc_set_lineattr(plot->pc, axis->line.line_width, 0, 3, 0);
+  gtk_plot_pc_draw_line(plot->pc,
                 x - size / 2.0 * axis->direction.x,
                 y - size / 2.0 * axis->direction.y,
                 x + axis->direction.x * size / 2.0,
                 y + axis->direction.y * size / 2.0);
   gtk_plot_pc_set_lineattr(plot->pc, axis->ticks_width, 0, 1, 0);
 
-  for(ntick = 0; ntick < axis->ticks.nticks; ntick++){
-    x_tick = axis->ticks.values[ntick].value;
-    if(axis->direction.x == 0)
-      gtk_plot_get_pixel(plot, x_tick, -polar->rotation+90, &xx, &yy);
-    else
-      gtk_plot_get_pixel(plot, x_tick, -polar->rotation, &yy, &xx);
-
-    yy = yy - y0;
-    if(!axis->ticks.values[ntick].minor && x_tick >= axis->ticks.min){
+  for(ntick = 0; ntick < axis->ticks.nmajorticks; ntick++){
+    xx = axis->ticks.major[ntick];
+    if(axis->ticks.major_values[ntick] >= axis->min){
       if(axis->major_mask & GTK_PLOT_TICKS_IN){
-         x1 = x + yy * axis->direction.x;
-         y1 = y + yy * axis->direction.y;
+         x1 = x + xx * axis->direction.x;
+         y1 = y + xx * axis->direction.y;
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
                        x1 + tick_direction.x * m * axis->ticks_length,
                        y1 + tick_direction.y * m * axis->ticks_length);
-         x1 = x - yy * axis->direction.x;
-         y1 = y - yy * axis->direction.y;
+         x1 = x - xx * axis->direction.x;
+         y1 = y - xx * axis->direction.y;
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
@@ -525,15 +472,15 @@ gtk_plot_polar_draw_axis(GtkPlotPolar *polar,
                        y1 + tick_direction.y * m * axis->ticks_length);
       }
       if(axis->major_mask & GTK_PLOT_TICKS_OUT){
-         x1 = x + yy * axis->direction.x;
-         y1 = y + yy * axis->direction.y;
+         x1 = x + xx * axis->direction.x;
+         y1 = y + xx * axis->direction.y;
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
                        x1 - tick_direction.x * m * axis->ticks_length,
                        y1 - tick_direction.y * m * axis->ticks_length);
-         x1 = x - yy * axis->direction.x;
-         y1 = y - yy * axis->direction.y;
+         x1 = x - xx * axis->direction.x;
+         y1 = y - xx * axis->direction.y;
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
@@ -541,17 +488,20 @@ gtk_plot_polar_draw_axis(GtkPlotPolar *polar,
                        y1 - tick_direction.y * m * axis->ticks_length);
       }
     }
-    if(axis->ticks.values[ntick].minor && x_tick >= axis->ticks.min){
+  }    
+  for(ntick = 0; ntick < axis->ticks.nminorticks; ntick++){
+    xx = axis->ticks.minor[ntick];
+    if(axis->ticks.minor_values[ntick] >= axis->min){
       if(axis->minor_mask & GTK_PLOT_TICKS_IN){
-         x1 = x + yy * axis->direction.x;
-         y1 = y + yy * axis->direction.y;
+         x1 = x + xx * axis->direction.x;
+         y1 = y + xx * axis->direction.y;
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
                        x1 + tick_direction.x * m * axis->ticks_length/2.,
                        y1 + tick_direction.y * m * axis->ticks_length/2.);
-         x1 = x - yy * axis->direction.x;
-         y1 = y - yy * axis->direction.y;
+         x1 = x - xx * axis->direction.x;
+         y1 = y - xx * axis->direction.y;
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
@@ -559,15 +509,15 @@ gtk_plot_polar_draw_axis(GtkPlotPolar *polar,
                        y1 + tick_direction.y * m * axis->ticks_length/2.);
       }
       if(axis->minor_mask & GTK_PLOT_TICKS_OUT){
-         x1 = x + yy * axis->direction.x;
-         y1 = y + yy * axis->direction.y;
+         x1 = x + xx * axis->direction.x;
+         y1 = y + xx * axis->direction.y;
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
                        x1 - tick_direction.x * m * axis->ticks_length/2.,
                        y1 - tick_direction.y * m * axis->ticks_length/2.);
-         x1 = x - yy * axis->direction.x;
-         y1 = y - yy * axis->direction.y;
+         x1 = x - xx * axis->direction.x;
+         y1 = y - xx * axis->direction.y;
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
@@ -588,29 +538,27 @@ gtk_plot_polar_draw_labels(GtkPlotPolar *polar,
   GtkWidget *widget;
   GtkPlot *plot;
   GdkFont *font;
-  GtkPSFont *psfont;
   GtkPlotText title, tick;
   gchar label[100];
   gdouble x_tick;
   gint x, y;
-  gdouble xx, yy;
+  gint xx, yy;
   gint ox, oy;
   gint text_height;
   gint xp, yp, width, height;
   gint ntick;
   gdouble m;
   gboolean veto = FALSE;
-  gdouble x0, y0;
 
   widget = GTK_WIDGET(polar);
   plot = GTK_PLOT(polar);
 
   m = plot->magnification;
 
-  xp = plot->internal_allocation.x;
-  yp = plot->internal_allocation.y;
-  width = plot->internal_allocation.width;
-  height = plot->internal_allocation.height;
+  xp = roundint(plot->x * widget->allocation.width);
+  yp = roundint(plot->y * widget->allocation.height);
+  width = roundint(plot->width * widget->allocation.width);
+  height = roundint(plot->height * widget->allocation.height);
 
   ox = width / 2.;
   oy = height / 2.;
@@ -620,8 +568,7 @@ gtk_plot_polar_draw_labels(GtkPlotPolar *polar,
 
   gtk_plot_pc_set_color(plot->pc, &axis->labels_attr.fg);
 
-  psfont = gtk_psfont_get_by_name(axis->labels_attr.font);
-  font = gtk_psfont_get_gdkfont(psfont,
+  font = gtk_psfont_get_gdkfont(axis->labels_attr.font,
                                 roundint(axis->labels_attr.height * m));
   text_height = font->ascent + font->descent;
 
@@ -638,32 +585,21 @@ gtk_plot_polar_draw_labels(GtkPlotPolar *polar,
            break;
   }
 
-  if(axis->direction.x == 0)
-    gtk_plot_get_pixel(plot, 0, -polar->rotation+90, &x0, &y0);
-  else
-    gtk_plot_get_pixel(plot, 0, -polar->rotation, &y0, &x0);
-
   tick = axis->labels_attr;
-  for(ntick = 0; ntick < axis->ticks.nticks; ntick++){
-    if(axis->ticks.values[ntick].minor) continue;
-    x_tick = axis->ticks.values[ntick].value;
-    if(axis->direction.x == 0)
-      gtk_plot_get_pixel(plot, x_tick, -polar->rotation+90, &xx, &yy);
-    else
-      gtk_plot_get_pixel(plot, x_tick, -polar->rotation, &yy, &xx);
-    yy = yy - y0;
-    xx = axis->direction.x * yy;
-    yy = axis->direction.y * yy;
-    if(x_tick >= axis->ticks.min-1.e-9){
+  for(ntick = 0; ntick < axis->ticks.nmajorticks; ntick++){
+    xx = axis->direction.x * axis->ticks.major[ntick];
+    yy = axis->direction.y * axis->ticks.major[ntick];
+    x_tick = axis->ticks.major_values[ntick];
+    if(x_tick >= axis->min-1.e-9){
       if(!axis->custom_labels){
-        gtk_plot_axis_parse_label(axis, x_tick, axis->label_precision, axis->label_style, label);
+        parse_label(x_tick, axis->label_precision, axis->label_style, label);
       }
       else
       {
         gtk_signal_emit_by_name(GTK_OBJECT(axis), "tick_label", 
                                 &x_tick, label, &veto);
         if(!veto)
-          gtk_plot_axis_parse_label(axis, x_tick, axis->label_precision, axis->label_style, label);
+          parse_label(x_tick, axis->label_precision, axis->label_style, label);
       }
       tick.text = label;
 
@@ -718,7 +654,6 @@ gtk_plot_polar_draw_circle(GtkPlotPolar *polar)
   GtkPlotAxis *axis, perp;
   GtkPlotText tick;
   GdkFont *font;
-  GtkPSFont *psfont;
   gchar label[100];
   gdouble x, y;
   gint line_width;
@@ -731,19 +666,17 @@ gtk_plot_polar_draw_circle(GtkPlotPolar *polar)
   gint text_height;
   gdouble rotation;
   gboolean veto = FALSE;
-  gint sign = 1;
 
   widget = GTK_WIDGET(polar);
   plot = GTK_PLOT(polar);
 
   m = plot->magnification;
   rotation = polar->rotation;
-  if(plot->reflect_y) sign = -1;
 
-  xp = plot->internal_allocation.x;
-  yp = plot->internal_allocation.y;
-  width = plot->internal_allocation.width;
-  height = plot->internal_allocation.height;
+  xp = widget->allocation.x + plot->x * widget->allocation.width;
+  yp = widget->allocation.y + plot->y * widget->allocation.height;
+  width = plot->width * widget->allocation.width;
+  height = plot->height * widget->allocation.height;
 
   ox = width / 2.;
   oy = height / 2.;
@@ -770,24 +703,23 @@ gtk_plot_polar_draw_circle(GtkPlotPolar *polar)
 
   gtk_plot_pc_set_lineattr(plot->pc, axis->ticks_width, 0, 1, 0);
 
-  psfont = gtk_psfont_get_by_name(axis->labels_attr.font);
-  font = gtk_psfont_get_gdkfont(psfont,
+  font = gtk_psfont_get_gdkfont(axis->labels_attr.font,
                                 roundint(axis->labels_attr.height * m));
   text_height = font->ascent + font->descent;
 
-  for(ntick = 0; ntick < axis->ticks.nticks; ntick++){
-    x_tick = axis->ticks.values[ntick].value;
-    if(!axis->ticks.values[ntick].minor && x_tick >= axis->ticks.min){
+  for(ntick = 0; ntick < axis->ticks.nmajorticks; ntick++){
+    if(axis->ticks.major_values[ntick] >= axis->min){
+      x_tick = axis->ticks.major_values[ntick];
 
       if(!axis->custom_labels){
-        gtk_plot_axis_parse_label(axis, x_tick, axis->label_precision, axis->label_style, label);
+        parse_label(x_tick, axis->label_precision, axis->label_style, label);
       }
       else
       {
         gtk_signal_emit_by_name(GTK_OBJECT(axis), "tick_label", 
                                 &x_tick, label, &veto);
         if(!veto)
-          gtk_plot_axis_parse_label(axis, x_tick, axis->label_precision, axis->label_style, label);
+          parse_label(x_tick, axis->label_precision, axis->label_style, label);
       }
 
       gtk_plot_get_pixel(plot,
@@ -798,8 +730,8 @@ gtk_plot_polar_draw_circle(GtkPlotPolar *polar)
       x_tick += rotation;
 
       if(axis->major_mask & GTK_PLOT_TICKS_IN){
-         perp.direction.x = cos(x_tick * PI / 180. * sign);
-         perp.direction.y = -sin(x_tick * PI / 180. * sign);
+         perp.direction.x = cos(x_tick * PI / 180.);
+         perp.direction.y = -sin(x_tick * PI / 180.);
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
@@ -807,8 +739,8 @@ gtk_plot_polar_draw_circle(GtkPlotPolar *polar)
                        y1 + perp.direction.y*m*axis->ticks_length);
       }
       if(axis->major_mask & GTK_PLOT_TICKS_OUT){
-         perp.direction.x = -cos(x_tick * PI / 180. * sign);
-         perp.direction.y = sin(x_tick * PI / 180. * sign);
+         perp.direction.x = -cos(x_tick * PI / 180.);
+         perp.direction.y = sin(x_tick * PI / 180.);
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
@@ -828,8 +760,8 @@ gtk_plot_polar_draw_circle(GtkPlotPolar *polar)
       y1 += text_height / 2;
 
       if((x_tick - rotation) != 360.0 && axis->label_mask != 0){
-         perp.direction.x = -cos(x_tick * PI / 180. * sign);
-         perp.direction.y = sin(x_tick * PI / 180. * sign);
+         perp.direction.x = -cos(x_tick * PI / 180.);
+         perp.direction.y = sin(x_tick * PI / 180.);
          tick.x = x1;
          tick.y = y1;
          tick.x -= roundint(perp.direction.x*axis->labels_offset * m);
@@ -840,7 +772,11 @@ gtk_plot_polar_draw_circle(GtkPlotPolar *polar)
       }
 
     }
-    if(axis->ticks.values[ntick].minor && x_tick >= axis->ticks.min){
+  }    
+
+  for(ntick = 0; ntick < axis->ticks.nminorticks; ntick++){
+    if(axis->ticks.minor_values[ntick] >= axis->min){
+      x_tick = axis->ticks.minor_values[ntick];
 
       gtk_plot_get_pixel(plot,
                          plot->ymax,
@@ -850,8 +786,8 @@ gtk_plot_polar_draw_circle(GtkPlotPolar *polar)
       x_tick += rotation;
 
       if(axis->minor_mask & GTK_PLOT_TICKS_IN){
-         perp.direction.x = cos(x_tick * PI / 180. * sign);
-         perp.direction.y = -sin(x_tick * PI / 180. * sign);
+         perp.direction.x = cos(x_tick * PI / 180.);
+         perp.direction.y = -sin(x_tick * PI / 180.);
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
@@ -859,8 +795,8 @@ gtk_plot_polar_draw_circle(GtkPlotPolar *polar)
                        y1 + perp.direction.y*m*axis->ticks_length/2.);
       }
       if(axis->minor_mask & GTK_PLOT_TICKS_OUT){
-         perp.direction.x = -cos(x_tick * PI / 180. * sign);
-         perp.direction.y = sin(x_tick * PI / 180. * sign);
+         perp.direction.x = -cos(x_tick * PI / 180.);
+         perp.direction.y = sin(x_tick * PI / 180.);
          gtk_plot_pc_draw_line(plot->pc,
                        x1,
                        y1,
@@ -874,6 +810,63 @@ gtk_plot_polar_draw_circle(GtkPlotPolar *polar)
 }
 
 
+static gint
+roundint (gdouble x)
+{
+ gint sign = 1;
+
+/* if(x <= 0.) sign = -1; 
+*/
+ return (x+sign*.50999999471);
+}
+
+static void
+parse_label(gdouble val, gint precision, gint style, gchar *label)
+{
+  gdouble auxval;
+  gint intspace = 0;
+  gint power;
+  gfloat v;
+
+  auxval = fabs(val);
+
+  power = 0.0;
+  if(auxval != 0.0)
+       power = (gint)log10(auxval);
+
+  v = val / pow(10.0, power); 
+  if(abs(v) < 1.0 && v != 0.0){
+     v *= 10.0;
+     power -= 1;
+  }
+  if(abs(v) >= 10.0){
+     v /= 10.0;
+     power += 1;
+  }
+  if(power < -12){
+     power = 0;
+     v =0.0;
+  }
+
+  if(auxval > 1)
+    intspace = (gint)log10(auxval);
+
+
+  switch(style){
+    case GTK_PLOT_LABEL_EXP:    
+      sprintf (label, "%*.*E", 1, precision, val);
+      break;
+    case GTK_PLOT_LABEL_POW:    
+      sprintf (label, "%*.*f\\4x\\N10\\S%i", intspace, precision, v, power);
+      break;
+    case GTK_PLOT_LABEL_FLOAT:
+    default:
+      sprintf (label, "%*.*f", intspace, precision, val);
+  }
+
+}
+
+
 static void
 gtk_plot_polar_real_get_pixel(GtkWidget *widget, 
                               gdouble x, gdouble y,
@@ -881,34 +874,29 @@ gtk_plot_polar_real_get_pixel(GtkWidget *widget,
 {
     GtkPlot *plot;
     GtkPlotPolar *polar;
-    gdouble xp, yp, width, height, size;
+    gdouble xp, yp, width, height;
     gdouble ox, oy;
-    gint sign = 1;
-    gdouble min;
+    gdouble rx, ry;
 
     plot = GTK_PLOT(widget);
     polar = GTK_PLOT_POLAR(widget);
 
-    if(plot->reflect_y) sign = -1;
-
-    xp = plot->internal_allocation.x;
-    yp = plot->internal_allocation.y;
-    width = plot->internal_allocation.width;
-    height = plot->internal_allocation.height;
-
-    size = MIN(width, height) / 2.;
+    xp = plot->x * widget->allocation.width;
+    yp = plot->y * widget->allocation.height;
+    width = plot->width * widget->allocation.width;
+    height = plot->height * widget->allocation.height;
 
     ox = xp + width / 2.0;
     oy = yp + height / 2.0;
 
-    min = plot->left->ticks.min;
-    plot->left->ticks.min = 0.0;
-    x = gtk_plot_axis_ticks_transform(plot->left, x)*size;
-    plot->left->ticks.min = min;
-
-    *px = ox + x * cos((y + polar->rotation) / 180. * PI * sign); 
-    *py = oy - x * sin((y + polar->rotation)/ 180. * PI * sign); 
+    rx = x * cos((y + polar->rotation) / 180. * PI); 
+    ry = x * sin((y + polar->rotation)/ 180. * PI); 
 
+    *px = transform(plot, rx);
+    *py = transform(plot, ry);
+   
+    *px += ox; 
+    *py = oy - *py; 
 }
 
 static void
@@ -924,10 +912,10 @@ gtk_plot_polar_real_get_point(GtkWidget *widget,
     gdouble rotation;
 
     plot = GTK_PLOT(widget);
-    xp = plot->internal_allocation.x;
-    yp = plot->internal_allocation.y;
-    width = plot->internal_allocation.width;
-    height = plot->internal_allocation.height;
+    xp = roundint(plot->x * widget->allocation.width);
+    yp = roundint(plot->y * widget->allocation.height);
+    width = roundint(plot->width * widget->allocation.width);
+    height = roundint(plot->height * widget->allocation.height);
 
     rotation = GTK_PLOT_POLAR(widget)->rotation;
 
@@ -959,10 +947,106 @@ gtk_plot_polar_real_get_point(GtkWidget *widget,
     r = sqrt(px * px + py * py);
 
     *x = 2.0 * r * plot->ymax / (gdouble) size;
-    *y = plot->reflect_y ? -angle : angle;
+    *y = angle;
 }
 
 
+/***********************************************************/
+
+static void
+gtk_plot_calc_ticks(GtkPlotPolar *polar, GtkPlotAxis *axis)
+{
+  GtkPlot *plot;
+  GtkPlotTicks *ticks = NULL;
+  GtkPlotScale scale;
+  gdouble min = 0., max = 0.;
+  gdouble absmin = 0., absmax = 0.;
+  gdouble pt = 0.0;
+  gdouble tick;
+  gdouble major_step;
+  gdouble minor_step;
+  gint nmajor, nminor;
+  gint n;
+
+  plot = GTK_PLOT(polar); 
+
+  scale = axis->scale;
+  ticks = &axis->ticks;
+
+  max = axis->max;
+  min = axis->min;
+
+  absmin = axis->min;
+  absmax = axis->max;
+
+  if(ticks->major != NULL){
+     g_free(ticks->major);
+     g_free(ticks->minor);
+     g_free(ticks->major_values);
+     g_free(ticks->minor_values);
+     ticks->major = NULL;
+     ticks->minor = NULL;
+     ticks->major_values = NULL;
+     ticks->minor_values = NULL;
+  }
+
+  nmajor = 0;
+  nminor = 0;
+  ticks->nmajorticks = 0;
+  ticks->nminorticks = 0;
+  major_step = ticks->step;
+  minor_step = major_step / ((gdouble)ticks->nminor + 1.0);
+
+  if(ticks->step > 0.){
+   tick = min - major_step;
+   while(tick <= absmax + 2*fabs(major_step)){
+     if(tick >= min-1.E-10 && tick <= absmax+1.E-10){
+        pt = transform(plot, tick);
+        nmajor ++;
+        ticks->major = (gint *)g_realloc(ticks->major, nmajor*sizeof(gint));
+        ticks->major_values = (gdouble *)g_realloc(ticks->major_values, nmajor*sizeof(gdouble));
+        ticks->major[nmajor-1] = pt;
+        ticks->major_values[nmajor-1] = tick;
+        ticks->nmajorticks = nmajor;
+     }
+     tick += major_step;
+   }
+  }
+
+  if(ticks->step >0. && ticks->nminor > 0){
+   for(nmajor = 0; nmajor < ticks->nmajorticks; nmajor++){
+    tick = ticks->major_values[nmajor];
+    for(n = 1; n <= ticks->nminor; n++){
+     tick += minor_step;
+     if(tick >= min-1.E-10 && tick <= absmax+1.E-10){
+        pt = transform(plot, tick);
+        nminor ++;
+        ticks->minor = (gint *)g_realloc(ticks->minor, nminor*sizeof(gint));
+        ticks->minor_values = (gdouble *)g_realloc(ticks->minor_values, nminor*sizeof(gdouble));
+        ticks->minor[nminor-1] = pt;
+        ticks->minor_values[nminor-1] = tick;
+        ticks->nminorticks = nminor;
+     }
+    }
+   }
+  }
+}
+
+static gdouble
+transform(GtkPlot *plot, gdouble y)
+{
+  gdouble width, height, size, position;
+
+  position = y / plot->ymax;
+
+  width = (gdouble)GTK_WIDGET(plot)->allocation.width * plot->width;
+  height = (gdouble)GTK_WIDGET(plot)->allocation.height * plot->height;
+
+  size = MIN(width, height) / 2.0;
+
+  return(size * position);
+}
+
 /*******************************************
  * gtk_plot_polar_rotate
  *******************************************/
@@ -976,9 +1060,3 @@ gtk_plot_polar_rotate(GtkPlotPolar *polar, gdouble angle)
 }
 
 
-gdouble
-gtk_plot_polar_get_angle(GtkPlotPolar *polar)
-{
-  return polar->rotation;
-}
-
