@@ -6,7 +6,6 @@
 #include "gtkplotdata.h"
 #include "gtkplotbox.h"
 #include "gtkplotcanvas.h"
-#include "gtkplotcanvasplot.h"
 #include "gtkplotps.h"
 #include "gtkplotprint.h"
 
@@ -45,7 +44,7 @@ gdouble gaussian(GtkPlot *plot, GtkPlotData *data, gdouble x, gboolean *err)
  return y;
 }
 
-/*
+
 gint
 select_item(GtkWidget *widget, GdkEvent *event, GtkPlotCanvasChild *item, 
             gpointer data)
@@ -92,9 +91,27 @@ select_item(GtkWidget *widget, GdkEvent *event, GtkPlotCanvasChild *item,
   widget_list = plots;
   active_widget = GTK_WIDGET(gtk_plot_canvas_get_active_plot(GTK_PLOT_CANVAS(widget)));
 
+  while(n < nlayers)
+    {
+      if(widget_list[n] == active_widget){
+            gtk_plot_canvas_set_active_plot(GTK_PLOT_CANVAS(canvas),
+                                            GTK_PLOT(plots[n]));  
+            GTK_BUTTON(buttons[n])->button_down = TRUE;
+            GTK_TOGGLE_BUTTON(buttons[n])->active = TRUE;
+            gtk_widget_set_state(buttons[n], GTK_STATE_ACTIVE);
+      }else{
+            GTK_BUTTON(buttons[n])->button_down = FALSE;
+            GTK_TOGGLE_BUTTON(buttons[n])->active = FALSE;
+            gtk_widget_set_state(buttons[n], GTK_STATE_NORMAL);
+      }
+      gtk_widget_queue_draw(buttons[n]);
+      
+      n++;
+    }
+
+
   return TRUE;
 }
-*/
 
 gint
 activate_plot(GtkWidget *widget, gpointer data)
@@ -110,14 +127,19 @@ activate_plot(GtkWidget *widget, gpointer data)
 
   while(n < nlayers)
     {
-      gtk_signal_handler_block_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
       if(widget_list[n] == active_widget){
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), TRUE);
+            gtk_plot_canvas_set_active_plot(GTK_PLOT_CANVAS(canvas),
+                                            GTK_PLOT(plots[n]));  
+            GTK_BUTTON(buttons[n])->button_down = TRUE;
+            GTK_TOGGLE_BUTTON(buttons[n])->active = TRUE;
+            gtk_widget_set_state(buttons[n], GTK_STATE_ACTIVE);
       }else{
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), FALSE);
+            GTK_BUTTON(buttons[n])->button_down = FALSE;
+            GTK_TOGGLE_BUTTON(buttons[n])->active = FALSE;
+            gtk_widget_set_state(buttons[n], GTK_STATE_NORMAL);
       }
-      gtk_signal_handler_unblock_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
-                                                                                
+      gtk_widget_queue_draw(buttons[n]);
+      
       n++;
     }
 
@@ -129,8 +151,6 @@ GtkWidget *
 new_layer(GtkWidget *canvas)
 {
  gchar label[10];
- GtkRequisition req;
- gint size;
 
  nlayers++;
 
@@ -142,9 +162,7 @@ new_layer(GtkWidget *canvas)
  buttons[nlayers-1] = gtk_toggle_button_new_with_label(label);
 /* gtk_button_set_relief(GTK_BUTTON(buttons[nlayers-1]), GTK_RELIEF_NONE);
 */
- gtk_widget_size_request(buttons[nlayers-1], &req);
- size = MAX(req.width,req.height);
- gtk_widget_set_usize(buttons[nlayers-1], size, size);
+ gtk_widget_set_usize(buttons[nlayers-1], 20, 20);
  gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*20, 0);
  gtk_widget_show(buttons[nlayers-1]);
 
@@ -164,11 +182,10 @@ void
 build_example1(GtkWidget *active_plot)
 {
  GdkColor color1, color2;
- static gdouble px[]={0., 0.2, 0.4, 0.6, 0.8, 1.0};
- static gdouble py[]={.243,.045,.075,.213,.05,.124};
- static gdouble pz[]={.56, .12, .123, .5, .2, .21};
- static gdouble dx[]={.10, .18, .17, .16, .082, .34};
- static gdouble dy[]={.1,.1,.1,.1,.1,.1};
+ static gdouble px1[]={0., 0.2, 0.4, 0.6, 0.8, 1.0};
+ static gdouble py1[]={.56, .12, .123, .5, .2, .21};
+ static gdouble pz1[]={.12, .22, .27, .12, .052, .42};
+ static gdouble dz1[]={.0243,.045,.075,.0213,.05,.0324};
 
  dataset[0] = GTK_PLOT_DATA(gtk_plot_box_new(GTK_ORIENTATION_VERTICAL));
 
@@ -179,8 +196,9 @@ build_example1(GtkWidget *active_plot)
 
  gtk_plot_add_data(GTK_PLOT(active_plot), dataset[0]);
  gtk_widget_show(GTK_WIDGET(dataset[0]));
- gtk_plot_data_set_points(dataset[0], px, py, dx, dy, 6);
- gtk_plot_data_set_z(dataset[0], pz);
+ gtk_plot_data_set_points(dataset[0], px1, py1, NULL, NULL, 6);
+ gtk_plot_data_set_z(dataset[0], pz1);
+ gtk_plot_data_set_dz(dataset[0], dz1);
 
  gtk_plot_data_show_zerrbars(dataset[0]);
 
@@ -195,7 +213,7 @@ build_example1(GtkWidget *active_plot)
                           10, 2, &color2, &color1);
  gtk_plot_data_set_line_attributes(dataset[0],
                                    GTK_PLOT_LINE_NONE,
-                                   0, 0, 1, &color1);
+                                   1, &color1);
 /*
  gtk_plot_data_set_x_attributes(dataset[0],
                                 GTK_PLOT_LINE_SOLID,
@@ -216,7 +234,6 @@ int main(int argc, char *argv[]){
  GtkWidget *scrollw1;
  GtkWidget *active_plot;
  GtkWidget *canvas;
- GtkPlotCanvasChild *child;
  gint page_width, page_height;
  gfloat scale = 1.;
  
@@ -260,26 +277,21 @@ int main(int argc, char *argv[]){
  gtk_plot_set_range(GTK_PLOT(active_plot), -1. ,1., -1., 1.4);
  gtk_plot_legends_move(GTK_PLOT(active_plot), .500, .05);
  gtk_plot_set_legends_border(GTK_PLOT(active_plot), 0, 0);
- gtk_plot_axis_hide_title(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP));
- gtk_plot_axis_show_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), 3), 15, 3);
- gtk_plot_axis_set_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), 0), 1., 1);
- gtk_plot_axis_set_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), 1), 1., 1);
- gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP), TRUE);
- gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT), TRUE);
+ gtk_plot_axis_hide_title(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP);
+ gtk_plot_axis_show_ticks(GTK_PLOT(active_plot), 3, 15, 3);
+ gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), 0, 1., 1);
+ gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), 1, 1., 1);
+ gtk_plot_axis_set_visible(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP, TRUE);
+ gtk_plot_axis_set_visible(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT, TRUE);
  gtk_plot_x0_set_visible(GTK_PLOT(active_plot), TRUE);
  gtk_plot_y0_set_visible(GTK_PLOT(active_plot), TRUE);
-
-
- child = gtk_plot_canvas_plot_new(GTK_PLOT(active_plot));
- gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .15, .06, .65, .31);
+ gtk_plot_canvas_add_plot(GTK_PLOT_CANVAS(canvas), GTK_PLOT(active_plot), .15, .06);
  gtk_widget_show(active_plot);
 
  build_example1(active_plot);
 
-/*
  gtk_signal_connect(GTK_OBJECT(canvas), "select_item",
                     (GtkSignalFunc) select_item, NULL);
-*/
 
  gtk_widget_show(window1);
 
