@@ -7,12 +7,6 @@
 #include "gtkplotdata.h"
 #include "gtkplotsurface.h"
 #include "gtkplotcanvas.h"
-#include "gtkplotcanvasplot.h"
-#include "gtkplotcanvastext.h"
-#include "gtkplotcanvasrectangle.h"
-#include "gtkplotcanvasellipse.h"
-#include "gtkplotcanvasline.h"
-#include "gtkplotcanvaspixmap.h"
 #include "gtkplotcsurface.h"
 #include "gtkplotps.h"
 #include "gtkplotprint.h"
@@ -23,7 +17,6 @@ GtkWidget **buttons;
 GtkPlotData *dataset[5];
 GtkPlot3D *plot3d;
 gint nlayers = 0;
-GtkWidget *active_plot;
 
 void
 quit ()
@@ -42,99 +35,11 @@ gdouble function(GtkPlot *plot,
  return z;
 }
 
-gint
-select_item(GtkWidget *widget, GdkEvent *event, GtkPlotCanvasChild *child, 
-            gpointer data)
-{
-  GtkWidget **widget_list = NULL;
-  GtkWidget *active_widget = NULL;
-  gint n = 0;
-  gdouble *x = NULL, *y = NULL;
-
-  if(GTK_IS_PLOT_CANVAS_TEXT(child))
-        printf("Item selected: TEXT\n");
-  if(GTK_IS_PLOT_CANVAS_PIXMAP(child))
-        printf("Item selected: PIXMAP\n");
-  if(GTK_IS_PLOT_CANVAS_RECTANGLE(child))
-        printf("Item selected: RECTANGLE\n");
-  if(GTK_IS_PLOT_CANVAS_ELLIPSE(child))
-        printf("Item selected: ELLIPSE\n");
-  if(GTK_IS_PLOT_CANVAS_LINE(child))
-        printf("Item selected: LINE\n");
-  if(GTK_IS_PLOT_CANVAS_PLOT(child)){
-    switch(GTK_PLOT_CANVAS_PLOT(child)->pos){
-      case GTK_PLOT_CANVAS_PLOT_IN_TITLE:
-        printf("Item selected: TITLE\n");
-        break;
-      case GTK_PLOT_CANVAS_PLOT_IN_LEGENDS:
-        printf("Item selected: LEGENDS\n");
-        break;
-      case GTK_PLOT_CANVAS_PLOT_IN_PLOT:
-        printf("Item selected: PLOT\n");
-        break;
-      case GTK_PLOT_CANVAS_PLOT_IN_AXIS:
-        printf("Item selected: AXIS\n");
-        break;
-      case GTK_PLOT_CANVAS_PLOT_IN_MARKER:
-        printf("Item selected: MARKER\n");
-        break;
-      case GTK_PLOT_CANVAS_PLOT_IN_GRADIENT:
-        printf("Item selected: GRADIENT\n");
-        break;
-      case GTK_PLOT_CANVAS_PLOT_IN_DATA:
-        x = gtk_plot_data_get_x(GTK_PLOT_CANVAS_PLOT(child)->data, &n); 
-        y = gtk_plot_data_get_y(GTK_PLOT_CANVAS_PLOT(child)->data, &n); 
-        n = GTK_PLOT_CANVAS_PLOT(child)->datapoint;
-        printf("Item selected: DATA\n");
-        printf("Active point: %d -> %f %f\n", 
-                GTK_PLOT_CANVAS_PLOT(child)->datapoint, x[n], y[n]);
-        break;
-      default:
-        break;
-    }
-
-    widget_list = plots;
-    active_widget = GTK_WIDGET(GTK_PLOT_CANVAS_PLOT(child)->plot);
-  }
-
-  return TRUE;
-}
-
-gint
-activate_plot(GtkWidget *widget, gpointer data)
-{
-  GtkWidget **widget_list = NULL;
-  GtkWidget *active_widget = NULL;
-  GtkWidget *canvas = NULL;
-  gint n = 0;
-
-  canvas = GTK_WIDGET(data);
-  widget_list = buttons;
-  active_widget = widget;
-
-  while(n < nlayers)
-    {
-      gtk_signal_handler_block_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
-      if(widget_list[n] == active_widget){
-            active_plot = plots[n];
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), TRUE);
-      }else{
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), FALSE);
-      }
-      gtk_signal_handler_unblock_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
-                                                                                
-      n++;
-    }
-
-  return TRUE;
-}
 
 GtkWidget *
 new_layer(GtkWidget *canvas, GtkType type)
 {
  gchar label[10];
- GtkRequisition req;
- gint size;
 
  nlayers++;
 
@@ -146,10 +51,8 @@ new_layer(GtkWidget *canvas, GtkType type)
  buttons[nlayers-1] = gtk_toggle_button_new_with_label(label);
 /* gtk_button_set_relief(GTK_BUTTON(buttons[nlayers-1]), GTK_RELIEF_NONE);
 */
- gtk_widget_size_request(buttons[nlayers-1], &req);
- size = MAX(req.width,req.height);
- gtk_widget_set_usize(buttons[nlayers-1], size, size);
- gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*size, 0);
+ gtk_widget_set_usize(buttons[nlayers-1], 20, 20);
+ gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*20, 0);
  gtk_widget_show(buttons[nlayers-1]);
 
  if(type == GTK_TYPE_PLOT3D)
@@ -165,9 +68,17 @@ new_layer(GtkWidget *canvas, GtkType type)
 void
 rotate_x(GtkWidget *button, gpointer data)
 {
- if(!GTK_IS_PLOT3D(active_plot)) return;
+ GtkPlot3D *active_plot;
+ GtkPlot *plot;
 
- gtk_plot3d_rotate_x(GTK_PLOT3D(active_plot), 10.);
+ plot = gtk_plot_canvas_get_active_plot(GTK_PLOT_CANVAS(data));
+
+ if(GTK_IS_PLOT3D(plot)) 
+     active_plot = GTK_PLOT3D(plot);
+ else
+     return;
+
+ gtk_plot3d_rotate_x(active_plot, 10.);
 
  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(data));
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(data));
@@ -176,9 +87,17 @@ rotate_x(GtkWidget *button, gpointer data)
 void
 rotate_y(GtkWidget *button, gpointer data)
 {
- if(!GTK_IS_PLOT3D(active_plot)) return;
+ GtkPlot3D *active_plot;
+ GtkPlot *plot;
 
- gtk_plot3d_rotate_y(GTK_PLOT3D(active_plot), 10.);
+ plot = gtk_plot_canvas_get_active_plot(GTK_PLOT_CANVAS(data));
+
+ if(GTK_IS_PLOT3D(plot)) 
+     active_plot = GTK_PLOT3D(plot);
+ else
+     return;
+
+ gtk_plot3d_rotate_y(active_plot, 10.);
 
  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(data));
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(data));
@@ -187,9 +106,17 @@ rotate_y(GtkWidget *button, gpointer data)
 void
 rotate_z(GtkWidget *button, gpointer data)
 {
- if(!GTK_IS_PLOT3D(active_plot)) return;
+ GtkPlot3D *active_plot;
+ GtkPlot *plot;
 
- gtk_plot3d_rotate_z(GTK_PLOT3D(active_plot), 10.);
+ plot = gtk_plot_canvas_get_active_plot(GTK_PLOT_CANVAS(data));
+
+ if(GTK_IS_PLOT3D(plot)) 
+     active_plot = GTK_PLOT3D(plot);
+ else
+     return;
+
+ gtk_plot3d_rotate_z(active_plot, 10.);
 
  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(data));
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(data));
@@ -201,10 +128,10 @@ int main(int argc, char *argv[]){
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *scrollw1;
+ GtkWidget *active_plot;
  GtkWidget *canvas;
  GtkWidget *button;
  GtkWidget *surface;
- GtkPlotCanvasChild *child;
  gint page_width, page_height;
  gfloat scale = 1.;
  gint n;
@@ -247,8 +174,7 @@ int main(int argc, char *argv[]){
  gtk_widget_show(canvas);
 
  plot3d = GTK_PLOT3D(new_layer(canvas, GTK_TYPE_PLOT3D));
- child = gtk_plot_canvas_plot_new(GTK_PLOT(plot3d));
- gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .16, .05, .65, .45);
+ gtk_plot_canvas_add_plot(GTK_PLOT_CANVAS(canvas), GTK_PLOT(plot3d), .16, .02);
  gtk_widget_show(GTK_WIDGET(plot3d));
 
  x = (gdouble *)g_malloc(1600 * sizeof(gdouble));
@@ -288,52 +214,57 @@ int main(int argc, char *argv[]){
 
 */
 
- gtk_plot3d_set_minor_ticks(GTK_PLOT3D(plot3d), GTK_PLOT_AXIS_X, 3);
- gtk_plot3d_set_minor_ticks(GTK_PLOT3D(plot3d), GTK_PLOT_AXIS_Y, 3);
+ gtk_plot3d_axis_set_minor_ticks(GTK_PLOT3D(plot3d), GTK_PLOT_AXIS_X, 3);
+ gtk_plot3d_axis_set_minor_ticks(GTK_PLOT3D(plot3d), GTK_PLOT_AXIS_Y, 3);
 
  gtk_plot3d_minor_grids_set_visible(GTK_PLOT3D(plot3d), FALSE, FALSE, FALSE);
 
 
  surface = gtk_plot_csurface_new_function((GtkPlotFunc3D) function);
 
- gtk_plot_surface_set_xstep(GTK_PLOT_SURFACE(surface), .05);
- gtk_plot_surface_set_ystep(GTK_PLOT_SURFACE(surface), .05);
+ gtk_plot_surface_set_xstep(GTK_PLOT_SURFACE(surface), .025);
+ gtk_plot_surface_set_ystep(GTK_PLOT_SURFACE(surface), .025);
+
+/*
+ gtk_plot_surface_set_xstep(GTK_PLOT_SURFACE(surface), .1);
+ gtk_plot_surface_set_ystep(GTK_PLOT_SURFACE(surface), .1);
+*/
 
  gtk_plot_data_set_legend(GTK_PLOT_DATA(surface), "cos ((r-r\\s0\\N)\\S2\\N)");
 
  gtk_plot_add_data(GTK_PLOT(plot3d), GTK_PLOT_DATA(surface));
  gtk_widget_show(surface);
 
- gtk_plot_data_set_gradient(GTK_PLOT_DATA(surface), .2, .8, 6, 0);
+ gtk_plot_data_set_gradient(GTK_PLOT_DATA(surface), .2, .8, 6, 3);
 /* 
  gtk_plot_surface_set_transparent(GTK_PLOT_SURFACE(surface), TRUE);
 */
- gtk_plot_surface_set_transparent(GTK_PLOT_SURFACE(surface), FALSE);
- gtk_plot_csurface_set_projection(GTK_PLOT_CSURFACE(surface), GTK_PLOT_PROJECT_EMPTY);
- GTK_PLOT_CSURFACE(surface)->levels_line.line_style = GTK_PLOT_LINE_NONE;
- GTK_PLOT_CSURFACE(surface)->sublevels_line.line_style = GTK_PLOT_LINE_NONE;
+ gtk_plot_csurface_set_lines_visible(GTK_PLOT_CSURFACE(surface), FALSE);
 /*******************/
  active_plot = new_layer(canvas, GTK_TYPE_PLOT);
- child = gtk_plot_canvas_plot_new(GTK_PLOT(active_plot));
- gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .26, .54, .65, .90);
+ gtk_plot_canvas_add_plot(GTK_PLOT_CANVAS(canvas), GTK_PLOT(active_plot), .26, .56);
 
- gtk_plot_axis_set_minor_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT), 1);
- gtk_plot_axis_set_minor_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM), 1);
+ gtk_plot_axis_set_minor_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT, 1);
+ gtk_plot_axis_set_minor_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM, 1);
 
  surface = gtk_plot_csurface_new_function((GtkPlotFunc3D) function);
 
- gtk_plot_surface_set_xstep(GTK_PLOT_SURFACE(surface), .050);
- gtk_plot_surface_set_ystep(GTK_PLOT_SURFACE(surface), .050);
+ gtk_plot_surface_set_xstep(GTK_PLOT_SURFACE(surface), .025);
+ gtk_plot_surface_set_ystep(GTK_PLOT_SURFACE(surface), .025);
+
+/*
+ gtk_plot_surface_set_xstep(GTK_PLOT_SURFACE(surface), .1);
+ gtk_plot_surface_set_ystep(GTK_PLOT_SURFACE(surface), .1);
+*/
 
  gtk_plot_data_set_legend(GTK_PLOT_DATA(surface), "cos ((r-r\\s0\\N)\\S2\\N)");
  gtk_plot_add_data(GTK_PLOT(active_plot), GTK_PLOT_DATA(surface));
  gtk_widget_show(surface);
 
- gtk_plot_data_set_gradient(GTK_PLOT_DATA(surface), .2, .8, 3, 0);
+ gtk_plot_data_set_gradient(GTK_PLOT_DATA(surface), .2, .8, 6, 0);
 
  gtk_plot_surface_set_grid_visible(GTK_PLOT_SURFACE(surface), FALSE);
- gtk_plot_surface_set_transparent(GTK_PLOT_SURFACE(surface), TRUE);
- gtk_plot_csurface_set_projection(GTK_PLOT_CSURFACE(surface), GTK_PLOT_PROJECT_FULL);
+ gtk_plot_csurface_set_lines_only(GTK_PLOT_CSURFACE(surface), TRUE);
 
 /*
  gtk_plot_csurface_set_lines_visible(GTK_PLOT_CSURFACE(surface), FALSE);
@@ -361,9 +292,6 @@ int main(int argc, char *argv[]){
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(rotate_z), canvas);
 
- gtk_signal_connect(GTK_OBJECT(canvas), "select_item",
-                    (GtkSignalFunc) select_item, NULL);
-                                                                                
  gtk_widget_show(window1);
 
  gtk_plot_canvas_export_ps(GTK_PLOT_CANVAS(canvas), "democsurface.ps", 0, 0,
