@@ -10,19 +10,12 @@
 #include "gtkplotcanvas.h"
 #include "gtkplotps.h"
 #include "gtkplotprint.h"
-#include "gtkplotcanvastext.h"
-#include "gtkplotcanvasline.h"
-#include "gtkplotcanvasellipse.h"
-#include "gtkplotcanvasrectangle.h"
-#include "gtkplotcanvasplot.h"
-#include "gtkplotcanvaspixmap.h"
 
 GdkPixmap *pixmap=NULL;
 GtkWidget **plots;
 GtkWidget **buttons;
 GtkPlotData *dataset[5];
 gint nlayers = 0;
-GtkWidget *active_plot;
 
 void
 quit ()
@@ -50,8 +43,6 @@ GtkWidget *
 new_layer(GtkWidget *canvas)
 {
  gchar label[10];
- GtkRequisition req;
- gint size;
 
  nlayers++;
 
@@ -63,9 +54,7 @@ new_layer(GtkWidget *canvas)
  buttons[nlayers-1] = gtk_toggle_button_new_with_label(label);
 /* gtk_button_set_relief(GTK_BUTTON(buttons[nlayers-1]), GTK_RELIEF_NONE);
 */
- gtk_widget_size_request(buttons[nlayers-1], &req);
- size = MAX(req.width,req.height);
- gtk_widget_set_usize(buttons[nlayers-1], size, size);
+ gtk_widget_set_usize(buttons[nlayers-1], 20, 20);
  gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*20, 0);
  gtk_widget_show(buttons[nlayers-1]);
 
@@ -79,7 +68,11 @@ new_layer(GtkWidget *canvas)
 void
 rotate_x(GtkWidget *button, gpointer data)
 {
- gtk_plot3d_rotate_x(GTK_PLOT3D(active_plot), 10.);
+ GtkPlot3D *active_plot;
+
+ active_plot = GTK_PLOT3D(gtk_plot_canvas_get_active_plot(GTK_PLOT_CANVAS(data)));
+
+ gtk_plot3d_rotate_x(active_plot, 10.);
 
  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(data));
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(data));
@@ -88,7 +81,11 @@ rotate_x(GtkWidget *button, gpointer data)
 void
 rotate_y(GtkWidget *button, gpointer data)
 {
- gtk_plot3d_rotate_y(GTK_PLOT3D(active_plot), 10.);
+ GtkPlot3D *active_plot;
+
+ active_plot = GTK_PLOT3D(gtk_plot_canvas_get_active_plot(GTK_PLOT_CANVAS(data)));
+
+ gtk_plot3d_rotate_y(active_plot, 10.);
 
  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(data));
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(data));
@@ -97,7 +94,11 @@ rotate_y(GtkWidget *button, gpointer data)
 void
 rotate_z(GtkWidget *button, gpointer data)
 {
- gtk_plot3d_rotate_z(GTK_PLOT3D(active_plot), 10.);
+ GtkPlot3D *active_plot;
+
+ active_plot = GTK_PLOT3D(gtk_plot_canvas_get_active_plot(GTK_PLOT_CANVAS(data)));
+
+ gtk_plot3d_rotate_z(active_plot, 10.);
 
  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(data));
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(data));
@@ -111,10 +112,11 @@ int main(int argc, char *argv[]){
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *scrollw1;
+ GtkWidget *active_plot;
  GtkWidget *canvas;
  GtkWidget *button;
  GtkWidget *surface;
- gdouble *x,*y,*z,*a;
+ gdouble *x,*y,*z;
  gint cnt= 0;
  gint page_width, page_height;
  gfloat scale = 1.;
@@ -127,8 +129,6 @@ int main(int argc, char *argv[]){
  gdouble zmin=1e99;
  gdouble zmax=-1e99;
  gdouble dx,dy,dz;
- gint n, nx, ny;
- GtkPlotCanvasChild *child;
  
  page_width = GTK_PLOT_LETTER_W * scale;
  page_height = GTK_PLOT_LETTER_H * scale;
@@ -165,14 +165,12 @@ int main(int argc, char *argv[]){
 
  active_plot = new_layer(canvas);
 
- child = gtk_plot_canvas_plot_new(GTK_PLOT(active_plot));
- gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .10, .06, .85, .85);
-
+ gtk_plot_canvas_add_plot(GTK_PLOT_CANVAS(canvas), GTK_PLOT(active_plot), .10, .06);
 
+/*
  x = (gdouble *)g_malloc(1600 * sizeof(gdouble));
  y = (gdouble *)g_malloc(1600 * sizeof(gdouble));
  z = (gdouble *)g_malloc(1600 * sizeof(gdouble));
- a = (gdouble *)g_malloc(1600 * sizeof(gdouble));
  n = 0;
 
  for(nx = 0; nx < 40; nx++)
@@ -181,7 +179,6 @@ int main(int argc, char *argv[]){
      x[n]=0.025*(gdouble)nx;
      y[n]=0.025*(gdouble)ny;
      z[n]=cos(((x[n]-0.5)*(x[n]-0.5) + (y[n]-0.5)*(y[n]-0.5))*24) / 4. + .5;
-     a[n]=((x[n]-0.5)*(x[n]-0.5) + (y[n]-0.5)*(y[n]-0.5));
      n++;
    }
 
@@ -190,11 +187,7 @@ int main(int argc, char *argv[]){
  gtk_plot_surface_set_points(GTK_PLOT_SURFACE(surface), 
                              x, y, z, NULL, NULL, NULL, 40, 40); 
  gtk_plot_data_set_legend(GTK_PLOT_DATA(surface), "cos ((r-r\\s0\\N)\\S2\\N)");
-
- gtk_plot_surface_use_amplitud(GTK_PLOT_SURFACE(surface), TRUE);
- gtk_plot_data_set_a(GTK_PLOT_DATA(surface), a);
- gtk_plot_data_gradient_set_visible(GTK_PLOT_DATA(surface), TRUE);
-
+*/
 /*
  for(nx = 0; nx < 10; nx++)
    for(ny = 0; ny < 10; ny++)
@@ -206,42 +199,39 @@ int main(int argc, char *argv[]){
    }
 
  surface = gtk_plot_surface_new();
-
  gtk_plot_surface_set_points(GTK_PLOT_SURFACE(surface), 
                              x, y, z, NULL, NULL, NULL, 10, 10); 
 */
 
- gtk_plot3d_set_minor_ticks(GTK_PLOT3D(active_plot), GTK_PLOT_AXIS_X, 1);
- gtk_plot3d_set_minor_ticks(GTK_PLOT3D(active_plot), GTK_PLOT_AXIS_Y, 1);
- gtk_plot3d_show_ticks(GTK_PLOT3D(active_plot), 
+ gtk_plot3d_axis_set_minor_ticks(GTK_PLOT3D(active_plot), GTK_PLOT_AXIS_X, 1);
+ gtk_plot3d_axis_set_minor_ticks(GTK_PLOT3D(active_plot), GTK_PLOT_AXIS_Y, 1);
+ gtk_plot3d_axis_show_ticks(GTK_PLOT3D(active_plot), 
                             GTK_PLOT_SIDE_XY, 
                             GTK_PLOT_TICKS_OUT, GTK_PLOT_TICKS_OUT);
- gtk_plot3d_show_ticks(GTK_PLOT3D(active_plot), 
+ gtk_plot3d_axis_show_ticks(GTK_PLOT3D(active_plot), 
                             GTK_PLOT_SIDE_XZ, 
                             GTK_PLOT_TICKS_OUT, GTK_PLOT_TICKS_OUT);
- gtk_plot3d_show_ticks(GTK_PLOT3D(active_plot), 
+ gtk_plot3d_axis_show_ticks(GTK_PLOT3D(active_plot), 
                             GTK_PLOT_SIDE_YX, 
                             GTK_PLOT_TICKS_OUT, GTK_PLOT_TICKS_OUT);
- gtk_plot3d_show_ticks(GTK_PLOT3D(active_plot), 
+ gtk_plot3d_axis_show_ticks(GTK_PLOT3D(active_plot), 
                             GTK_PLOT_SIDE_YZ, 
                             GTK_PLOT_TICKS_OUT, GTK_PLOT_TICKS_OUT);
- gtk_plot3d_show_ticks(GTK_PLOT3D(active_plot), 
+ gtk_plot3d_axis_show_ticks(GTK_PLOT3D(active_plot), 
                             GTK_PLOT_SIDE_ZX, 
                             GTK_PLOT_TICKS_OUT, GTK_PLOT_TICKS_OUT);
- gtk_plot3d_show_ticks(GTK_PLOT3D(active_plot), 
+ gtk_plot3d_axis_show_ticks(GTK_PLOT3D(active_plot), 
                             GTK_PLOT_SIDE_ZY, 
                             GTK_PLOT_TICKS_OUT, GTK_PLOT_TICKS_OUT);
 
  /* if a file was specified on the commandline, try to open and display it: */
 
  if (argc!=2 || !(f=fopen(argv[1],"r"))) {
-/*
    x= y= z= NULL;
    surface = gtk_plot_surface_new_function((GtkPlotFunc3D) function);
    gtk_plot_surface_set_xstep(GTK_PLOT_SURFACE(surface), .025);
    gtk_plot_surface_set_ystep(GTK_PLOT_SURFACE(surface), .025);
    gtk_plot_data_set_legend(GTK_PLOT_DATA(surface), "cos ((r-r\\s0\\N)\\S2\\N)");
-*/
    gtk_plot_add_data(GTK_PLOT(active_plot), GTK_PLOT_DATA(surface));
  } else {
    x= g_new(gdouble,MAXNODES);
@@ -270,9 +260,9 @@ int main(int argc, char *argv[]){
    gtk_plot3d_set_xrange(GTK_PLOT3D(active_plot), xmin-dx, xmax+dx);
    gtk_plot3d_set_yrange(GTK_PLOT3D(active_plot), ymin-dy, ymax+dy);
    gtk_plot3d_set_zrange(GTK_PLOT3D(active_plot), zmin-dz, zmax+dz);
-   gtk_plot3d_set_ticks(GTK_PLOT3D(active_plot), GTK_PLOT_AXIS_X, (xmax-xmin)/10., 1);
-   gtk_plot3d_set_ticks(GTK_PLOT3D(active_plot), GTK_PLOT_AXIS_Y, (ymax-ymin)/10., 1);
-   gtk_plot3d_set_ticks(GTK_PLOT3D(active_plot), GTK_PLOT_AXIS_Z, (zmax-zmin)/10., 1);
+   gtk_plot3d_axis_set_ticks(GTK_PLOT3D(active_plot), GTK_PLOT_AXIS_X, (xmax-xmin)/10., 1);
+   gtk_plot3d_axis_set_ticks(GTK_PLOT3D(active_plot), GTK_PLOT_AXIS_Y, (ymax-ymin)/10., 1);
+   gtk_plot3d_axis_set_ticks(GTK_PLOT3D(active_plot), GTK_PLOT_AXIS_Z, (zmax-zmin)/10., 1);
    */
 
    surface = gtk_plot_csurface_new();
@@ -287,21 +277,7 @@ int main(int argc, char *argv[]){
  gtk_plot_data_set_gradient(GTK_PLOT_DATA(surface), .2, .8, 6, 0);
 */
 
-/*
- gtk_plot_data_gradient_set_scale(GTK_PLOT_DATA(surface),GTK_PLOT_SCALE_LOG10);
-*/
-
  gtk_plot3d_autoscale(GTK_PLOT3D(active_plot));
-/*
- gtk_plot3d_set_xrange(GTK_PLOT3D(active_plot), -0.5,1);
- gtk_plot3d_set_yrange(GTK_PLOT3D(active_plot), -0.2,1);
- gtk_plot3d_set_zrange(GTK_PLOT3D(active_plot), 0.,1);
-*/
-/*
- gtk_plot3d_rotate_x(GTK_PLOT3D(active_plot), GTK_PLOT3D(active_plot)->a1+360-300);
- gtk_plot3d_rotate_y(GTK_PLOT3D(active_plot), GTK_PLOT3D(active_plot)->a2+360-360);
- gtk_plot3d_rotate_z(GTK_PLOT3D(active_plot), GTK_PLOT3D(active_plot)->a3+360-330);
-*/
  gtk_widget_show(surface);
 
 
@@ -322,8 +298,6 @@ int main(int argc, char *argv[]){
 
 /*
  gtk_plot3d_set_xfactor(GTK_PLOT3D(active_plot), 0.5);
-*/
-/*
  gtk_plot3d_set_xrange(GTK_PLOT3D(active_plot), 0., 2.0);
 */
 
@@ -340,8 +314,6 @@ int main(int argc, char *argv[]){
  gtk_plot_canvas_export_ps_with_size(GTK_PLOT_CANVAS(canvas), "demo3d.ps", 
 				     0, 0, GTK_PLOT_PSPOINTS,
                             	     GTK_PLOT_LETTER_W, GTK_PLOT_LETTER_H);
-
-
  gtk_main();
 
  return(0);
