@@ -5,9 +5,8 @@
 #include <gdk/gdk.h>
 #include "gtkplot.h"
 #include "gtkplotdata.h"
-#include "gtkplotsegment.h"
+#include "gtkplotflux.h"
 #include "gtkplotcanvas.h"
-#include "gtkplotcanvasplot.h"
 #include "gtkplotps.h"
 #include "gtkplotprint.h"
 #include "gtkplotdt.h"
@@ -17,7 +16,6 @@ GtkWidget **plots;
 GtkWidget **buttons;
 GtkPlotData *dataset[5];
 gint nlayers = 0;
-GtkWidget *active_plot;
 
 
 void
@@ -40,27 +38,30 @@ activate_plot(GtkWidget *widget, gpointer data)
 
   while(n < nlayers)
     {
-      gtk_signal_handler_block_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
       if(widget_list[n] == active_widget){
-            active_plot = plots[n];
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), TRUE);
+            gtk_plot_canvas_set_active_plot(GTK_PLOT_CANVAS(canvas),
+                                            GTK_PLOT(plots[n]));  
+            GTK_BUTTON(buttons[n])->button_down = TRUE;
+            GTK_TOGGLE_BUTTON(buttons[n])->active = TRUE;
+            gtk_widget_set_state(buttons[n], GTK_STATE_ACTIVE);
       }else{
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), FALSE);
+            GTK_BUTTON(buttons[n])->button_down = FALSE;
+            GTK_TOGGLE_BUTTON(buttons[n])->active = FALSE;
+            gtk_widget_set_state(buttons[n], GTK_STATE_NORMAL);
       }
-      gtk_signal_handler_unblock_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
-                                                                                
+      gtk_widget_queue_draw(buttons[n]);
+      
       n++;
     }
 
   return TRUE;
 }
 
+
 GtkWidget *
 new_layer(GtkWidget *canvas)
 {
  gchar label[10];
- GtkRequisition req;
- gint size;
 
  nlayers++;
 
@@ -72,9 +73,7 @@ new_layer(GtkWidget *canvas)
  buttons[nlayers-1] = gtk_toggle_button_new_with_label(label);
 /* gtk_button_set_relief(GTK_BUTTON(buttons[nlayers-1]), GTK_RELIEF_NONE);
 */
- gtk_widget_size_request(buttons[nlayers-1], &req);
- size = MAX(req.width,req.height);
- gtk_widget_set_usize(buttons[nlayers-1], size, size);
+ gtk_widget_set_usize(buttons[nlayers-1], 20, 20);
  gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*20, 0);
  gtk_widget_show(buttons[nlayers-1]);
 
@@ -113,36 +112,35 @@ build_example1(GtkWidget *active_plot,GtkPlotDT *data, gint num)
    cx= gtk_plot_dt_get_node(data,t->c)->x;
    cy= gtk_plot_dt_get_node(data,t->c)->y;
    px1[i*3+0]= ax;    py1[i*3+0]= ay;
-   px2[i*3+0]= bx; py2[i*3+0]= by;
+   px2[i*3+0]= bx-ax; py2[i*3+0]= by-ay;
    px1[i*3+1]= bx;    py1[i*3+1]= by;
-   px2[i*3+1]= cx; py2[i*3+1]= cy;
+   px2[i*3+1]= cx-bx; py2[i*3+1]= cy-by;
    px1[i*3+2]= cx;    py1[i*3+2]= cy;
-   px2[i*3+2]= ax; py2[i*3+2]= ay;
+   px2[i*3+2]= ax-cx; py2[i*3+2]= ay-cy;
 
    i++;
    list = list->next;
  }
  fprintf(stderr,"%d triangles, %d points\n",num,num*3);
 
- dataset[0] = GTK_PLOT_DATA(gtk_plot_segment_new());
- gtk_plot_segment_set_relative(GTK_PLOT_SEGMENT(dataset[0]), FALSE);
+ dataset[0] = GTK_PLOT_DATA(gtk_plot_flux_new());
  gtk_plot_add_data(GTK_PLOT(active_plot), dataset[0]);
  gtk_widget_show(GTK_WIDGET(dataset[0]));
  gtk_plot_data_set_points(dataset[0], px1, py1, px2, py2, num*3);
 
- gtk_plot_segment_center(GTK_PLOT_SEGMENT(dataset[0]), FALSE);
+ gtk_plot_flux_center(GTK_PLOT_FLUX(dataset[0]), FALSE);
 
 
  gdk_color_parse("red", &color);
  gdk_color_alloc(gdk_colormap_get_system(), &color); 
- gtk_plot_segment_set_arrow(GTK_PLOT_SEGMENT(dataset[0]), 0, 0, 0);
+ gtk_plot_flux_set_arrow(GTK_PLOT_FLUX(dataset[0]), 0, 0, 0);
  gtk_plot_data_set_symbol(dataset[0],
                           GTK_PLOT_SYMBOL_CIRCLE,
 			  GTK_PLOT_SYMBOL_OPAQUE,
                           0,1, &color, &color);
  gtk_plot_data_set_line_attributes(dataset[0],
                                    GTK_PLOT_LINE_NONE,
-                                   0, 0, 0, &color);
+                                   0, &color);
 
  gtk_plot_data_set_legend(dataset[0], "Delaunay triangulization");
 
@@ -185,8 +183,8 @@ int main(int argc, char *argv[]){
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *scrollw1;
+ GtkWidget *active_plot;
  GtkWidget *canvas;
- GtkPlotCanvasChild *child;
  gint page_width, page_height;
  gfloat scale = 1.;
  GtkPlotDTnode p;
@@ -206,7 +204,7 @@ int main(int argc, char *argv[]){
  gtk_init(&argc,&argv);
 
  window1=gtk_window_new(GTK_WINDOW_TOPLEVEL);
- gtk_window_set_title(GTK_WINDOW(window1), "GtkPlotDT Demo");
+ gtk_window_set_title(GTK_WINDOW(window1), "GtkPlotFlux Demo");
  gtk_widget_set_usize(window1,550,650);
  gtk_container_border_width(GTK_CONTAINER(window1),0);
 
@@ -274,25 +272,22 @@ int main(int argc, char *argv[]){
  gtk_plot_set_range(GTK_PLOT(active_plot), xmin-dx, xmax+dx, ymin-dy, ymax+dy);
  gtk_plot_legends_move(GTK_PLOT(active_plot), .500, .05);
  gtk_plot_set_legends_border(GTK_PLOT(active_plot), 0, 0);
- gtk_plot_axis_hide_title(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP));
- gtk_plot_axis_show_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), 3), 15, 3);
- gtk_plot_axis_set_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), 0), (xmax-xmin)/10., 1);
- gtk_plot_axis_set_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), 1), (ymax-ymin)/10., 1);
- gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP), TRUE);
- gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT), TRUE);
+ gtk_plot_axis_hide_title(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP);
+ gtk_plot_axis_show_ticks(GTK_PLOT(active_plot), 3, 15, 3);
+ gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), 0, (xmax-xmin)/10., 1);
+ gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), 1, (ymax-ymin)/10., 1);
+ gtk_plot_axis_set_visible(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP, TRUE);
+ gtk_plot_axis_set_visible(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT, TRUE);
  gtk_plot_x0_set_visible(GTK_PLOT(active_plot), TRUE);
  gtk_plot_y0_set_visible(GTK_PLOT(active_plot), TRUE);
-
-
- child = gtk_plot_canvas_plot_new(GTK_PLOT(active_plot));
- gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .15, .06, .65, .65);
+ gtk_plot_canvas_add_plot(GTK_PLOT_CANVAS(canvas), GTK_PLOT(active_plot), .15, .06);
  gtk_widget_show(active_plot);
 
  build_example1(active_plot,dtdata, num_triangles);
 
  gtk_widget_show(window1);
 
- gtk_plot_canvas_export_ps(GTK_PLOT_CANVAS(canvas), "demodtsegment.ps", 0, 0, 
+ gtk_plot_canvas_export_ps(GTK_PLOT_CANVAS(canvas), "demodtflux.ps", 0, 0, 
                            GTK_PLOT_LETTER);
  
  gtk_main();
