@@ -6,7 +6,6 @@
 #include "gtkplotdata.h"
 #include "gtkplotpixmap.h"
 #include "gtkplotcanvas.h"
-#include "gtkplotcanvasplot.h"
 #include "gtkplotps.h"
 #include "gtkplotprint.h"
 
@@ -28,8 +27,6 @@ GtkWidget *
 new_layer(GtkWidget *canvas)
 {
  gchar label[10];
- GtkRequisition req;
- gint size;
 
  nlayers++;
 
@@ -41,9 +38,7 @@ new_layer(GtkWidget *canvas)
  buttons[nlayers-1] = gtk_toggle_button_new_with_label(label);
 /* gtk_button_set_relief(GTK_BUTTON(buttons[nlayers-1]), GTK_RELIEF_NONE);
 */
- gtk_widget_size_request(buttons[nlayers-1], &req);
- size = MAX(req.width,req.height);
- gtk_widget_set_usize(buttons[nlayers-1], size, size);
+ gtk_widget_set_usize(buttons[nlayers-1], 20, 20);
  gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*20, 0);
  gtk_widget_show(buttons[nlayers-1]);
 
@@ -101,7 +96,6 @@ int main(int argc, char *argv[]){
  GtkWidget *canvas;
  gint page_width, page_height;
  gfloat scale = 1.;
- GtkPlotCanvasChild *child;
  
  page_width = GTK_PLOT_LETTER_W * scale;
  page_height = GTK_PLOT_LETTER_H * scale;
@@ -143,18 +137,15 @@ int main(int argc, char *argv[]){
  gtk_plot_set_range(GTK_PLOT(active_plot), 0. ,1., 0., 1.4);
  gtk_plot_legends_move(GTK_PLOT(active_plot), .500, .05);
  gtk_plot_set_legends_border(GTK_PLOT(active_plot), 0, 0);
- gtk_plot_axis_hide_title(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP));
- gtk_plot_axis_show_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), 3), 15, 3);
- gtk_plot_axis_set_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), 0), 1., 1);
- gtk_plot_axis_set_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), 1), 1., 1);
- gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP), TRUE);
- gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT), TRUE);
+ gtk_plot_axis_hide_title(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP);
+ gtk_plot_axis_show_ticks(GTK_PLOT(active_plot), 3, 15, 3);
+ gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), 0, 1., 1);
+ gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), 1, 1., 1);
+ gtk_plot_axis_set_visible(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP, TRUE);
+ gtk_plot_axis_set_visible(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT, TRUE);
  gtk_plot_x0_set_visible(GTK_PLOT(active_plot), TRUE);
  gtk_plot_y0_set_visible(GTK_PLOT(active_plot), TRUE);
-
-
- child = gtk_plot_canvas_plot_new(GTK_PLOT(active_plot));
- gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .15, .06, .65, .35);
+ gtk_plot_canvas_add_plot(GTK_PLOT_CANVAS(canvas), GTK_PLOT(active_plot), .15, .06);
  gtk_widget_show(active_plot);
 
  build_example1(active_plot);
